# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import datetime
from unittest import mock
from lxml import etree

from superdesk.utc import utcnow
from superdesk.tests import TestCase
from superdesk.publish import init_app
from superdesk.publish.formatters.newsml_1_2_formatter import NewsML12Formatter


@mock.patch('superdesk.publish.subscribers.SubscribersService.generate_sequence_number', lambda self, subscriber: 1)
class Newsml12FormatterTest(TestCase):
    article = {
        '_id': 'urn:localhost.abc',
        'guid': 'urn:localhost.abc',
        'source': 'AAP',
        'anpa_category': [{'qcode': 'a', 'name': 'Australian General News'}],
        'headline': 'This is a test headline - для теста!',
        'byline': 'joe',
        'slugline': 'slugline',
        'subject': [{'qcode': '02011001'}, {'qcode': '02011002'}],
        'anpa_take_key': 'take_key',
        'unique_id': '1',
        'body_html': '<p>The story body&nbsp;more to the story - больше историй</p>',
        'type': 'text',
        'word_count': '1',
        'priority': 1,
        '_current_version': 5,
        'state': 'published',
        'urgency': 2,
        'pubstatus': 'usable',
        'dateline': {
            'source': 'AAP',
            'text': 'sample dateline',
            'located': {
                'alt_name': '',
                'state': 'California',
                'city_code': 'Los Angeles',
                'city': 'Los Angeles',
                'dateline': 'city',
                'country_code': 'US',
                'country': 'USA',
                'tz': 'America/Los_Angeles',
                'state_code': 'CA'
            }
        },
        'keywords': ['traffic'],
        'abstract': 'sample abstract',
        'place': [
            {'qcode': 'NSW', 'name': 'NSW', 'state': 'New South Wales',
             'country': 'Australia', 'world_region': 'Oceania'}
        ],
        'ednote': 'this is test',
        'body_footer': '<p>call helpline 999 if you are planning to quit smoking</p>',
        'company_codes': [{'name': 'YANCOAL AUSTRALIA LIMITED', 'qcode': 'YAL', 'security_exchange': 'ASX'}]
    }

    preformatted = {
        '_id': 'urn:localhost.123',
        'guid': 'urn:localhost.123',
        'source': 'AAP',
        'anpa_category': [{'qcode': 'a', 'name': 'Australian General News'}],
        'headline': 'This is a test headline - для теста!',
        'byline': 'joe',
        'slugline': 'slugline',
        'subject': [{'qcode': '02011001'}, {'qcode': '02011002'}],
        'anpa_take_key': 'take_key',
        'unique_id': '1',
        'type': 'preformatted',
        'body_html': 'The story body',
        'word_count': '1',
        '_current_version': 5,
        'state': 'published',
        'urgency': 2,
        'pubstatus': 'usable',
        'dateline': {'text': 'sample dateline'},
        'keywords': ['traffic'],
        'abstract': 'sample abstract',
        'place': [
            {'qcode': 'Australia', 'name': 'Australia', 'state': '',
             'country': 'Australia', 'world_region': 'Oceania'}
        ]
    }

    picture = {
        '_id': 'tag:localhost:2015:cf15b059-b997-4e34-a103-85b8d7ea4ba3',
        'guid': 'tag:localhost:2015:cf15b059-b997-4e34-a103-85b8d7ea4ba3',
        'firstcreated': '2015-09-20T06:12:57.000Z',
        'versioncreated': '2015-09-20T06:14:11.000Z',
        'dateline': {
            'source': 'AAP',
            'date': '2015-08-14T04:45:53.000Z'
        },
        'renditions': {
            'viewImage': {
                'height': 415,
                'href': 'http://localhost:5000/api/upload/55cd72811d41c828e1773786/raw?_schema=http',
                'media': '55cd72811d41c828e1773786',
                'mimetype': 'image/jpeg',
                'width': 640
            },
            'baseImage': {
                'height': 909,
                'href': 'http://localhost:5000/api/upload/55cd72811d41c828e1773782/raw?_schema=http',
                'media': '55cd72811d41c828e1773782',
                'mimetype': 'image/jpeg',
                'width': 1400
            },
            'thumbnail': {
                'height': 120,
                'href': 'http://localhost:5000/api/upload/55cd72811d41c828e1773784/raw?_schema=http',
                'media': '55cd72811d41c828e1773784',
                'mimetype': 'image/jpeg',
                'width': 184
            },
            'original': {
                'height': 2455,
                'href': 'http://localhost:5000/api/upload/55cd72801d41c828e1773762/raw?_schema=http',
                'media': '55cd72801d41c828e1773762',
                'mimetype': 'image/jpeg',
                'width': 3777
            }
        },
        'state': 'published',
        'anpa_category': [{'qcode': 'a', 'name': 'Australian General News'}],
        'source': 'AAP Image',
        '_current_version': 1,
        'original_source': 'AAP Image/AAP',
        'description': 'Federal Education Minister Christopher Pyne launches his new book NO ARCHIVING',
        'type': 'picture',
        'slugline': 'NUS CHRISTOPHER PYNE PROTEST',
        'headline': 'NUS CHRISTOPHER PYNE PROTEST',
        'pubstatus': 'usable',
        'ednote': '',
        'byline': 'TRACEY NEARMY',
        'filemeta': {
            'yresolution': [
                300,
                1
            ],
            'exposuretime': [
                1,
                200
            ],
            'copyright': '                                                      ',
            'scenecapturetype': 0,
            'sensingmethod': 2,
            'fnumber': [
                14,
                5
            ],
            'flashpixversion': '0100',
            'xresolution': [
                300,
                1
            ],
            'resolutionunit': 2,
            'subsectimedigitized': '20',
            'exposureprogram': 1,
            'subsectimeoriginal': '20',
            'make': 'NIKON CORPORATION',
            'focallengthin35mmfilm': 200,
            'scenetype': 1,
            'exifimageheight': 2455,
            'saturation': 0,
            'colorspace': 1,
            'subjectdistancerange': 0,
            'datetime': '2015:07:31 18:55:37',
            'software': 'Photogene for iPad v4.3',
            'flash': 16,
            'focallength': [
                200,
                1
            ],
            'componentsconfiguration': '\u0001\u0002\u0003\u0000',
            'lightsource': 3,
            'artist': '                                    ',
            'isospeedratings': 2000,
            'whitepoint': [
                313,
                1000
            ],
            'sharpness': 2,
            'exposuremode': 1,
            'meteringmode': 3,
            'compressedbitsperpixel': [
                4,
                1
            ],
            'model': 'NIKON D800E',
            'subsectime': '20',
            'datetimedigitized': '2015:07:31 18:55:37',
            'exifoffset': 406,
            'contrast': 0,
            'whitebalance': 1,
            'exifimagewidth': 3777,
            'datetimeoriginal': '2015:07:31 18:55:37',
            'customrendered': 0,
            'maxaperturevalue': [
                3,
                1
            ],
            'digitalzoomratio': [
                1,
                1
            ],
            'primarychromaticities': [
                16,
                25
            ],
            'length': 8009209,
            'exifversion': '0230',
            'gaincontrol': 2,
            'gamma': [
                11,
                5
            ],
            'filesource': 3
        },
        'language': 'en',
        'mimetype': 'image/jpeg',
        'sign_off': 'mar',
        'unique_id': 573
    }

    video = {
        '_id': 'urn:newsml:localhost:2015-09-20T16:12:57.333001:f3856812-0999-4ed8-b69e-68dcdeb1ed2e',
        'guid': 'tag:localhost:2015:c11e11c4-cdbc-41ef-b939-2b30dd8365fb',
        'language': 'en',
        'family_id': 'urn:newsml:localhost:2015-09-20T16:12:57.333001:f3856812-0999-4ed8-b69e-68dcdeb1ed2e',
        '_current_version': 3,
        'versioncreated': '2015-09-20T06:14:11.000Z',
        'unique_id': 274,
        'renditions': {
            'original': {
                'media': '55fe4e691d41c8cac923ceb2',
                'href': 'http://192.168.220.176:5000/api/upload/55fe4e691d41c8cac923ceb2/raw?_schema=http',
                'mimetype': 'video/mp4'
            }
        },
        'state': 'in_progress',
        'version_creator': '55ee82871d41c86ee1d78c45',
        'sign_off': 'ADM',
        'media': '55fe4e691d41c8cac923ceb2',
        'source': 'AAP',
        'original_source': 'AAP Video/AAP',
        'pubstatus': 'usable',
        'filemeta': {
            'mime_type': 'video/mp4',
            'last_modification': '1904-01-01T00:00:00+00:00',
            'creation_date': '1904-01-01T00:00:00+00:00',
            'height': '270',
            'width': '480',
            'duration': '0:00:10.224000',
            'comment': 'User volume: 100.0%',
            'length': 877869,
            'endian': 'Big endian'
        },
        'event_id': 'tag:localhost:2015:f3ae4441-4721-4987-8265-88d747b6a550',
        'original_creator': '55ee82871d41c86ee1d78c45',
        'expiry': '2016-12-21T14:14:11.000Z',
        'firstcreated': '2015-09-20T06:12:57.000Z',
        '_created': '2015-09-20T06:12:57.000Z',
        'type': 'video',
        'unique_name': '#274',
        'mimetype': 'video/mp4',
        'version': 2,
        'headline': 'test video',
        'description': 'test video',
        'abstract': 'test video',
        'slugline': 'test video keyword',
        'byline': 'test video',
        'subject': [
            {
                'qcode': '01001000',
                'name': 'archaeology',
                'parent': '01000000'
            }
        ],
        'place': [
            {
                'qcode': 'ACT',
                'name': 'ACT'
            }
        ],
        'anpa_category': [
            {
                'qcode': 'a',
                'name': 'Australian General News'
            }
        ]
    }

    vocab = [{'_id': 'rightsinfo', 'items': [{'name': 'AAP',
                                              'copyrightHolder': 'copy right holder',
                                              'copyrightNotice': 'copy right notice',
                                              'usageTerms': 'terms'},
                                             {'name': 'default',
                                              'copyrightHolder': 'default copy right holder',
                                              'copyrightNotice': 'default copy right notice',
                                              'usageTerms': 'default terms'}]}]

    package = {
        '_id': 'urn:newsml:localhost:2015-08-12T11:59:58.457029:7e90d257-92f6-406d-9186-95653b211701',
        'guid': 'urn:newsml:localhost:2015-08-12T11:59:58.457029:7e90d257-92f6-406d-9186-95653b211701',
        'type': 'composite',
        '_current_version': 1,
        'groups': [
            {
                'role': 'grpRole:NEP',
                'id': 'root',
                'refs': [
                    {
                        'idRef': 'main'
                    }
                ]
            },
            {
                'role': 'grpRole:main',
                'id': 'main',
                'refs': [
                    {
                        'type': 'text',
                        'renditions': {},
                        'itemClass': 'icls:text',
                        'guid': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'residRef': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'location': 'archive',
                        'headline': 'US:US cop sacked over student shooting',
                        'slugline': 'US Police',
                        '_current_version': 4
                    }
                ]
            }
        ],
        'pubstatus': 'usable',
        'state': 'published',
        'marked_for_not_publication': False,
        'dateline': {
            'located': {
                'alt_name': '',
                'state': 'California',
                'city_code': 'Los Angeles',
                'city': 'Los Angeles',
                'dateline': 'city',
                'country_code': 'US',
                'country': 'USA',
                'tz': 'America/Los_Angeles',
                'state_code': 'CA'
            },
            'date': '2015-08-12T01:59:58.000Z',
            'source': 'AAP',
            'text': 'Los Angeles, Aug 11 AAP -'
        },
        'language': 'en',
        'headline': 'Cop sacked over student shooting',
        'source': 'AAP',
        'slugline': 'US Police',
        'anpa_category': [
            {
                'name': 'International News',
                'qcode': 'I'
            }
        ],
        'subject': [
            {
                'name': 'police',
                'parent': '02000000',
                'qcode': '02003000'
            }
        ]
    }

    picture_package = {
        '_id': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'guid': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'language': 'en',
        'pubstatus': 'usable',
        'groups': [
            {
                'refs': [
                    {
                        'idRef': 'main'
                    }
                ],
                'id': 'root',
                'role': 'grpRole:NEP'
            },
            {
                'refs': [
                    {
                        'guid': '20150813001165688150',
                        'headline': 'Prison Riot',
                        'residRef': 'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037',
                        'location': 'archive',
                        'type': 'picture',
                        'slugline': 'Prison Riot',
                        'renditions': {
                            'baseImage': {
                                'height': 1400,
                                'mimetype': 'image/jpeg',
                                'width': 1120,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650a'
                            },
                            'thumbnail': {
                                'height': 120,
                                'mimetype': 'image/jpeg',
                                'width': 96,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650c'
                            },
                            'viewImage': {
                                'height': 640,
                                'mimetype': 'image/jpeg',
                                'width': 512,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650e'
                            },
                            'original': {
                                'height': 800,
                                'mimetype': 'image/jpeg',
                                'width': 640,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b6508'
                            }
                        },
                        'itemClass': 'icls:picture',
                        '_current_version': 4
                    }
                ],
                'id': 'main',
                'role': 'grpRole:main'
            }
        ],
        'type': 'composite',
        'state': 'published',
        'slugline': 'Prison Riot',
        'description': 'This Jan. 21, 2015 photo is of something)',
        'source': 'AAP',
        'headline': 'Prison Riot',
        '_current_version': 1,
        'dateline': {
            'date': '2015-08-13T04:07:59.000Z',
            'source': 'AAP'
        },
        'marked_for_not_publication': False,
        'sign_off': 'mar',
    }

    picture_text_package = {
        '_id': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'guid': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'language': 'en',
        'pubstatus': 'usable',
        'groups': [
            {
                'refs': [
                    {
                        'idRef': 'main'
                    }
                ],
                'id': 'root',
                'role': 'grpRole:NEP'
            },
            {
                'refs': [
                    {
                        'type': 'text',
                        'renditions': {},
                        'itemClass': 'icls:text',
                        'guid': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'residRef': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'location': 'archive',
                        'headline': 'US:US cop sacked over student shooting',
                        'slugline': 'US Police',
                        '_current_version': 4
                    },
                    {
                        'guid': '20150813001165688150',
                        'headline': 'Prison Riot',
                        'residRef': 'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037',
                        'location': 'archive',
                        'type': 'picture',
                        'slugline': 'Prison Riot',
                        'renditions': {
                            'baseImage': {
                                'height': 1400,
                                'mimetype': 'image/jpeg',
                                'width': 1120,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650a'
                            },
                            'thumbnail': {
                                'height': 120,
                                'mimetype': 'image/jpeg',
                                'width': 96,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650c'
                            },
                            'viewImage': {
                                'height': 640,
                                'mimetype': 'image/jpeg',
                                'width': 512,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650e'
                            },
                            'original': {
                                'height': 800,
                                'mimetype': 'image/jpeg',
                                'width': 640,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b6508'
                            }
                        },
                        'itemClass': 'icls:picture',
                        '_current_version': 4
                    }
                ],
                'id': 'main',
                'role': 'grpRole:main'
            }
        ],
        'type': 'composite',
        'state': 'published',
        'slugline': 'Prison Riot',
        'description': 'This Jan. 21, 2015 photo is of something)',
        'source': 'AAP',
        'headline': 'Prison Riot',
        '_current_version': 1,
        'dateline': {
            'date': '2015-08-13T04:07:59.000Z',
            'source': 'AAP'
        },
        'marked_for_not_publication': False,
        'sign_off': 'mar',
    }

    picture_text_package_multi_group = {
        '_id': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'guid': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'language': 'en',
        'pubstatus': 'usable',
        'groups': [
            {
                'refs': [
                    {'idRef': 'main'}, {'idRef': 'picture'}
                ],
                'id': 'root',
                'role': 'grpRole:NEP'
            },
            {
                'refs': [
                    {
                        'type': 'text',
                        'renditions': {},
                        'itemClass': 'icls:text',
                        'guid': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'residRef': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'location': 'archive',
                        'headline': 'US:US cop sacked over student shooting',
                        'slugline': 'US Police',
                        '_current_version': 4
                    }
                ],
                'id': 'main',
                'role': 'grpRole:main'
            },
            {
                'refs': [
                    {
                        'guid': '20150813001165688150',
                        'headline': 'Prison Riot',
                        'residRef': 'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037',
                        'location': 'archive',
                        'type': 'picture',
                        'slugline': 'Prison Riot',
                        'renditions': {
                            'baseImage': {
                                'height': 1400,
                                'mimetype': 'image/jpeg',
                                'width': 1120,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650a'
                            },
                            'thumbnail': {
                                'height': 120,
                                'mimetype': 'image/jpeg',
                                'width': 96,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650c'
                            },
                            'viewImage': {
                                'height': 640,
                                'mimetype': 'image/jpeg',
                                'width': 512,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650e'
                            },
                            'original': {
                                'height': 800,
                                'mimetype': 'image/jpeg',
                                'width': 640,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b6508'
                            }
                        },
                        'itemClass': 'icls:picture',
                        '_current_version': 4
                    }
                ],
                'id': 'picture',
                'role': 'grpRole:picture'
            }
        ],
        'type': 'composite',
        'state': 'published',
        'slugline': 'Prison Riot',
        'description': 'This Jan. 21, 2015 photo is of something)',
        'source': 'AAP',
        'headline': 'Prison Riot',
        '_current_version': 1,
        'dateline': {
            'date': '2015-08-13T04:07:59.000Z',
            'source': 'AAP'
        },
        'marked_for_not_publication': False,
        'sign_off': 'mar',
    }

    packaged_articles = [{'_id': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                          'headline': 'package article headline',
                          'slugline': 'slugline',
                          '_current_version': 4,
                          'state': 'published',
                          'pubStatus': 'usable'},
                         {'_id': 'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037',
                          'headline': 'package article headline',
                          'slugline': 'slugline',
                          '_current_version': 4,
                          'state': 'published',
                          'pubStatus': 'usable'}]

    now = datetime.datetime(2015, 6, 13, 11, 45, 19, 0)

    def setUp(self):
        self.article['state'] = 'published'
        self._setup_dates([self.article, self.video, self.picture,
                           self.package, self.picture_package,
                           self.preformatted, self.picture_text_package,
                           self.picture_text_package_multi_group])
        self.newsml = etree.Element("NewsML")
        self.formatter = NewsML12Formatter()
        self.formatter.now = self.now
        self.formatter.string_now = self.now.strftime('%Y%m%dT%H%M%S+0000')
        with self.app.app_context():
            init_app(self.app)
            self.app.data.insert('vocabularies', self.vocab)

    def _setup_dates(self, item_list):
        for item in item_list:
            item['firstcreated'] = self.now
            item['versioncreated'] = self.now

    def test_format_news_envelope(self):
        self.formatter._format_news_envelope(self.article, self.newsml, 7)
        self.assertEqual(self.newsml.find('TransmissionId').text, '7')
        self.assertEqual(self.newsml.find('DateAndTime').text, '20150613T114519+0000')
        self.assertEqual(self.newsml.find('Priority').get('FormalName'), '1')
        newsml = etree.Element("NewsML")
        self.formatter._format_news_envelope(self.preformatted, newsml, 7)
        self.assertEqual(newsml.find('Priority').get('FormalName'), '5')

    def test_format_identification(self):
        self.formatter._format_identification(self.article, self.newsml)
        self.assertEqual(self.newsml.find('Identification/NewsIdentifier/ProviderId').text, 'sourcefabric.org')
        self.assertEqual(self.newsml.find('Identification/NewsIdentifier/DateId').text, '20150613')
        self.assertEqual(self.newsml.find('Identification/NewsIdentifier/NewsItemId').text, 'urn:localhost.abc')
        self.assertEqual(self.newsml.find('Identification/NewsIdentifier/RevisionId').get('PreviousRevision'), '0')
        self.assertEqual(self.newsml.find('Identification/NewsIdentifier/PublicIdentifier').text,
                         'urn:localhost.abc:5N')
        self.assertEqual(self.newsml.find('Identification/DateLabel').text, 'Saturday 13 June 2015')

    def test_format_identification_for_corrections(self):
        self.article['state'] = 'corrected'
        self.article['_current_version'] = 7
        self.formatter._format_identification(self.article, self.newsml)
        self.assertEqual(self.newsml.find('Identification/NewsIdentifier/RevisionId').get('PreviousRevision'), '6')
        self.assertEqual(self.newsml.find('Identification/NewsIdentifier/RevisionId').get('Update'), 'N')
        self.article['state'] = 'killed'
        self.formatter._format_identification(self.article, self.newsml)
        self.assertEqual(self.newsml.find('Identification/NewsIdentifier/RevisionId').get('PreviousRevision'), '6')
        self.assertEqual(self.newsml.find('Identification/NewsIdentifier/RevisionId').get('Update'), 'N')

    def test_format_news_management(self):
        self.formatter._format_news_management(self.article, self.newsml)
        self.assertEqual(self.newsml.find('NewsManagement/NewsItemType').get('FormalName'), 'News')
        self.assertEqual(self.newsml.find('NewsManagement/FirstCreated').text, '20150613T114519+0000')
        self.assertEqual(self.newsml.find('NewsManagement/ThisRevisionCreated').text, '20150613T114519+0000')
        self.assertEqual(self.newsml.find('NewsManagement/Status').get('FormalName'), 'usable')
        self.assertEqual(self.newsml.find('NewsManagement/Urgency').get('FormalName'), '2')
        self.assertEqual(self.newsml.find('NewsManagement/Instruction').get('FormalName'), 'Update')

    def test_format_news_management_for_corrections(self):
        self.article['state'] = 'corrected'
        self.formatter._format_news_management(self.article, self.newsml)
        self.assertEqual(self.newsml.find('NewsManagement/Instruction').get('FormalName'), 'Correction')

    def test_format_news_component(self):
        self.formatter._format_news_component(self.article, self.newsml)
        self.assertEqual(self.newsml.find('NewsComponent/NewsComponent/Role').
                         get('FormalName'), 'Main')
        self.assertEqual(self.newsml.find('NewsComponent/NewsComponent/NewsLines/HeadLine').
                         text, 'This is a test headline - для теста!')
        self.assertEqual(self.newsml.find('NewsComponent/NewsComponent/NewsLines/ByLine').
                         text, 'joe')
        self.assertEqual(self.newsml.find('NewsComponent/NewsComponent/NewsLines/DateLine').
                         text, 'sample dateline')
        self.assertEqual(self.newsml.find('NewsComponent/NewsComponent/NewsLines/CreditLine').
                         text, 'AAP')
        self.assertEqual(self.newsml.find('NewsComponent/NewsComponent/NewsLines/KeywordLine').
                         text, 'slugline')
        self.assertEqual(
            self.newsml.findall(
                'NewsComponent/NewsComponent/DescriptiveMetadata/SubjectCode/Subject')[0].get('FormalName'), '02011001')
        self.assertEqual(
            self.newsml.findall(
                'NewsComponent/NewsComponent/DescriptiveMetadata/SubjectCode/Subject')[1].get('FormalName'), '02011002')
        self.assertEqual(self.newsml.find(
            'NewsComponent/NewsComponent/DescriptiveMetadata/Property').get('Value'), 'a')
        self.assertEqual(
            self.newsml.findall(
                'NewsComponent/NewsComponent/NewsComponent/ContentItem/DataContent')[0].text, 'sample abstract')
        self.assertEqual(self.newsml.findall(
            'NewsComponent/NewsComponent/NewsComponent/ContentItem/DataContent/nitf/body/body.content/p')[0].text,
            'The story body\xa0more to the story - больше историй')
        self.assertEqual(
            self.newsml.findall(
                'NewsComponent/NewsComponent/NewsComponent/ContentItem/DataContent/nitf/body/body.content/p')[1].text,
            'call helpline 999 if you are planning to quit smoking')
        self.assertEqual(self.newsml.find('.//NewsLines/NewsLine/NewsLineText').text, 'this is test')

        company_info = self.newsml.find('NewsComponent/NewsComponent/Metadata/Property[@FormalName="Ticker Symbol"]')
        self.assertEqual(company_info.attrib['Value'], 'YAL')

        company_info = self.newsml.find('NewsComponent/NewsComponent/Metadata/Property[@FormalName="Exchange"]')
        self.assertEqual(company_info.attrib['Value'], 'ASX')

    def test_format_news_management_for_embargo(self):
        embargo_ts = (utcnow() + datetime.timedelta(days=2))
        doc = self.article.copy()
        doc['embargo'] = embargo_ts

        self.formatter._format_news_management(doc, self.newsml)

        self.assertEqual(self.newsml.find('NewsManagement/NewsItemType').get('FormalName'), 'News')
        self.assertEqual(self.newsml.find('NewsManagement/FirstCreated').text, '20150613T114519+0000')
        self.assertEqual(self.newsml.find('NewsManagement/ThisRevisionCreated').text, '20150613T114519+0000')
        self.assertEqual(self.newsml.find('NewsManagement/Urgency').get('FormalName'), '2')
        self.assertEqual(self.newsml.find('NewsManagement/Instruction').get('FormalName'), 'Update')
        self.assertEqual(self.newsml.find('NewsManagement/Status').get('FormalName'), 'Embargoed')
        self.assertEqual(self.newsml.find('NewsManagement/StatusWillChange/FutureStatus').get('FormalName'), 'usable')
        self.assertEqual(self.newsml.find('NewsManagement/StatusWillChange/DateAndTime').text, embargo_ts.isoformat())

    def test_format_place(self):
        doc = self.article.copy()
        self.formatter._format_place(doc, self.newsml)
        self.assertEqual(self.newsml.find(
            'Location/Property[@FormalName="CountryArea"]').get('Value'), "New South Wales")
        self.assertEqual(self.newsml.find('Location/Property[@FormalName="Country"]').get('Value'), "Australia")
        self.assertEqual(self.newsml.find('Location/Property[@FormalName="WorldRegion"]').get('Value'), "Oceania")

    def test_format_dateline(self):
        doc = self.article.copy()
        self.formatter._format_dateline(doc, self.newsml)
        self.assertEqual(self.newsml.find('Location/Property[@FormalName="City"]').get('Value'), "Los Angeles")
        self.assertEqual(self.newsml.find('Location/Property[@FormalName="CountryArea"]').get('Value'), "California")
        self.assertEqual(self.newsml.find('Location/Property[@FormalName="Country"]').get('Value'), "USA")

    def test_duration(self):
        self.assertEqual(self.formatter._get_total_duration(None), 0)
        self.assertEqual(self.formatter._get_total_duration('dsf'), 0)
        self.assertEqual(self.formatter._get_total_duration('0:1:0.0000'), 60)
        self.assertEqual(self.formatter._get_total_duration('0:1:10.0000'), 70)
        self.assertEqual(self.formatter._get_total_duration('1:1:10.0000'), 3670)

    def test_format_picture(self):
        doc = self.picture.copy()
        seq, xml_str = self.formatter.format(doc, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(xml_str)

        self.assertEqual(xml.find('NewsItem/NewsComponent/NewsComponent/NewsLines/HeadLine').text,
                         'NUS CHRISTOPHER PYNE PROTEST')
        self.assertEqual(xml.find('NewsItem/NewsComponent/NewsComponent/NewsLines/ByLine').text, 'TRACEY NEARMY')
        self.assertEqual(xml.find('NewsItem/NewsComponent/NewsComponent/NewsLines/CreditLine').text, 'AAP Image/AAP')
        self.assertEqual(xml.find('NewsItem/NewsComponent/NewsComponent/NewsLines/KeywordLine').text,
                         'NUS CHRISTOPHER PYNE PROTEST')
        self.assertEqual(xml.find(('NewsItem/NewsComponent/NewsComponent/DescriptiveMetadata/'
                                   'Property[@FormalName="Category"]')).get('Value'), 'a')

        for rendition, value in doc.get('renditions').items():
            xpath = './/Role[@FormalName="{}"]/../ContentItem'.format(rendition)
            content_item = xml.find(xpath)
            self.assertEqual(content_item.get('Href'), value.get('href'))
            self.assertEqual(content_item.find('MediaType').get('FormalName'), 'Photo')
            self.assertEqual(content_item.find('Format').get('FormalName'), value.get('mimetype'))
            self.assertEqual(content_item.find('Characteristics/Property[@FormalName="Width"]').get('Value'),
                             str(value.get('width')))
            self.assertEqual(content_item.find('Characteristics/Property[@FormalName="Height"]').get('Value'),
                             str(value.get('height')))

    def test_format_video(self):
        doc = self.video.copy()
        seq, xml_str = self.formatter.format(doc, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(xml_str)
        self.assertEqual(xml.find('NewsItem/NewsComponent/NewsComponent/NewsLines/HeadLine').text, 'test video')
        self.assertEqual(xml.find('NewsItem/NewsComponent/NewsComponent/NewsLines/ByLine').text, 'test video')
        self.assertEqual(xml.find('NewsItem/NewsComponent/NewsComponent/NewsLines/CreditLine').text, 'AAP Video/AAP')
        self.assertEqual(xml.find('NewsItem/NewsComponent/NewsComponent/NewsLines/KeywordLine').text,
                         'test video keyword')
        for rendition, value in doc.get('renditions').items():
            xpath = './/Role[@FormalName="{}"]/../ContentItem'.format(rendition)
            content_item = xml.find(xpath)
            self.assertEqual(content_item.get('Href'), value.get('href'))
            self.assertEqual(content_item.find('MediaType').get('FormalName'), 'Video')
            self.assertEqual(content_item.find('Format').get('FormalName'), value.get('mimetype'))
            self.assertEqual(content_item.find('Characteristics/Property[@FormalName="Width"]').get('Value'),
                             str(doc.get('filemeta', {}).get('width')))
            self.assertEqual(content_item.find('Characteristics/Property[@FormalName="Height"]').get('Value'),
                             str(doc.get('filemeta', {}).get('height')))
            self.assertEqual(content_item.find('Characteristics/Property[@FormalName="TotalDuration"]').get('Value'),
                             '10')

    def test_format_package(self):
        doc = self.package.copy()
        seq, xml_str = self.formatter.format(doc, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(xml_str)
        self.assertEqual(xml.find('.//Role[@FormalName="root"]/../NewsComponent/Role').get('FormalName'),
                         'grpRole:main')
        self.assertEqual(xml.find('.//Role[@FormalName="root"]/../NewsComponent/NewsItemRef').get('NewsItem'),
                         'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b:4N')

    def test_format_picture_package(self):
        doc = self.picture_package.copy()
        seq, xml_str = self.formatter.format(doc, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(xml_str)
        self.assertEqual(xml.find('.//Role[@FormalName="root"]/../NewsComponent/Role').get('FormalName'),
                         'grpRole:main')
        self.assertEqual(xml.find('.//Role[@FormalName="root"]/../NewsComponent/NewsItemRef').get('NewsItem'),
                         'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037:4N')

    def test_format_picture_text_package(self):
        doc = self.picture_text_package.copy()
        seq, xml_str = self.formatter.format(doc, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(xml_str)
        news_component = xml.find('.//Role[@FormalName="root"]/../NewsComponent')
        self.assertEqual(news_component.find('Role').get('FormalName'), 'grpRole:main')
        news_item_refs = news_component.findall('NewsItemRef')
        self.assertEqual(news_item_refs[0].get('NewsItem'),
                         'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b:4N')
        self.assertEqual(news_item_refs[1].get('NewsItem'),
                         'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037:4N')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.metadata.utils import item_url
from superdesk.metadata.item import metadata_schema
from superdesk.auth_server.scopes import Scope


# NOTE: since schema is not defined here, setting up a projection explicitly is required,
# otherwise default `eve` fields (projection) will be applied e.q. `{'_id': 1}`
# and it will cut off all required data.
# https://github.com/pyeve/eve/blob/afd573d9254a9a23393f35760e9c515300909ccd/eve/io/base.py#L420
projection = {key: 1 for key in metadata_schema}
projection.update({
    'fields_meta': 0,
})


class ItemsResource(Resource):
    url = 'items'
    item_url = item_url
    item_methods = ['GET']
    resource_methods = ['GET']
    datasource = {
        'source': 'archive',
        'search_backend': 'elastic',
        'default_sort': [('_updated', -1)],
        'projection': projection,
    }
    privileges = {
        'GET': Scope.ARCHIVE_READ.name
    }




from superdesk.tests import TestCase
from superdesk.vocabularies.commands import get_vocabularies, is_changed, update_item


class VocabulariesUpdateArchiveTestCase(TestCase):
    fields = ['subject', 'genre', 'place', 'anpa_category']
    vocabularies_list = [
        {
            '_id': 'country_custom',
            'items': [
                {
                    'name': 'Australia',
                    'qcode': 'country_custom:1001001',
                    'is_active': True,
                    'translations': {
                        'name': {
                            'de': 'Australien',
                            'it': 'Australia',
                            'ja': 'オーストラリア'
                        }
                    }
                },
                {
                    'name': 'Austria',
                    'qcode': 'country_custom:1001002',
                    'is_active': True,
                    'translations': {
                        'name': {
                            'de': 'Österreich',
                            'it': 'Austria',
                            'ja': 'オーストリア'
                        }
                    }
                }
            ]
        },

        {
            '_id': 'subject_custom',
            'items': [
                {
                    'name': 'Asset allocation',
                    'parent': None,
                    'is_active': True,
                    'qcode': 'subject:01000000',
                    'translations': {
                        'name': {
                            'de': 'Asset-Allokation',
                            'it': 'Asset allocation',
                            'ja': 'アセット・アロケーション'
                        }
                    }
                },
                {
                    'name': 'Risk',
                    'parent': None,
                    'is_active': True,
                    'qcode': 'subject:02000000',
                    'translations': {
                        'name': {
                            'de': 'Risiko',
                            'it': 'Rischio',
                            'ja': 'リスク'
                        }
                    }
                },
                {
                    'name': 'Market context',
                    'parent': None,
                    'is_active': True,
                    'qcode': 'subject:03000000',
                    'translations': {
                        'name': {
                            'de': 'Marktumfeld',
                            'it': 'Contesto di mercato',
                            'ja': 'マーケット・コンテクスト'
                        }
                    }
                }
            ]
        }
    ]

    def test_get_vocabularies(self):
        vocabularies = get_vocabularies(self.vocabularies_list)

        self.assertTrue('country_custom' in vocabularies)
        self.assertTrue('values' in vocabularies['country_custom'])
        self.assertTrue('country_custom:1001001' in vocabularies['country_custom']['values'])
        self.assertTrue('country_custom:1001002' in vocabularies['country_custom']['values'])
        self.assertTrue(
            vocabularies['country_custom']['values']['country_custom:1001001']['scheme'] == 'country_custom'
        )
        self.assertTrue(
            vocabularies['country_custom']['values']['country_custom:1001002']['scheme'] == 'country_custom'
        )

        self.assertTrue('subject_custom' in vocabularies)
        self.assertTrue('values' in vocabularies['subject_custom'])
        self.assertTrue('values' in vocabularies['subject_custom'])
        self.assertTrue('subject:01000000' in vocabularies['subject_custom']['values'])
        self.assertTrue('subject:02000000' in vocabularies['subject_custom']['values'])
        self.assertTrue('subject:03000000' in vocabularies['subject_custom']['values'])
        self.assertTrue(vocabularies['subject_custom']['values']['subject:01000000']['scheme'] == 'subject_custom')
        self.assertTrue(vocabularies['subject_custom']['values']['subject:02000000']['scheme'] == 'subject_custom')
        self.assertTrue(vocabularies['subject_custom']['values']['subject:03000000']['scheme'] == 'subject_custom')

    def test_is_changed(self):
        old = {
            'name': 'Market context',
            'parent': None,
            'qcode': 'subject:03000000',
            'translations': {'name': {'de': 'Marktumfeld', 'it': 'Contesto di mercato', 'ja': 'マーケット・コンテクスト'}}
        }

        new = {
            'name': 'Market context',
            'parent': None,
            'qcode': 'subject:03000000',
            'translations': {'name': {'de': 'changed', 'it': 'Contesto di mercato', 'ja': 'マーケット・コンテクスト'}}
        }

        new_no_translations = {
            'name': 'Market context',
            'parent': None,
            'qcode': 'subject:03000000'
        }

        self.assertTrue(is_changed(old, {}))
        self.assertTrue(is_changed(old, {'name': 'Market context'}))
        self.assertTrue(is_changed(old, new_no_translations))
        self.assertFalse(is_changed(old, old))
        self.assertTrue(is_changed(old, new))

    def test_update_item(self):
        vocabularies = get_vocabularies(self.vocabularies_list)

        item = {
            'place': [
                self.vocabularies_list[0]['items'][0],
                self.vocabularies_list[0]['items'][0]
            ],
            'subject': [
                self.vocabularies_list[1]['items'][1],
                self.vocabularies_list[1]['items'][1]
            ]
        }

        item['subject'][0]['scheme'] = 'country_custom'
        item['subject'][0]['scheme'] = 'country_custom'
        item['place'][0]['scheme'] = 'subject_custom'
        item['place'][0]['scheme'] = 'subject_custom'
        changed = {'qcode': 'subject:03000000', 'scheme': 'subject_custom'}
        item['subject'].append(changed)

        updates = update_item(item, vocabularies, self.fields)

        self.assertTrue('subject' in updates)
        self.assertTrue(updates['subject'][0].get('qcode') == 'subject:03000000')
        self.assertTrue('translations' in updates['subject'][0])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.errors import SuperdeskApiError
from superdesk.services import BaseService
from flask_babel import _


class KeywordsService(BaseService):
    """Analyze the text and extract the keywords"""

    def __init__(self, datasource=None, backend=None):
        super().__init__(datasource, backend)
        self.provider = None

    def create(self, docs, **kwargs):
        if not self.provider:
            raise SuperdeskApiError.internalError(_('Not set a keywords provider'))

        try:
            ids = []
            for doc in docs:
                doc['keywords'] = self.provider.get_keywords(doc.get('text', ''))
                ids.append(len(ids))
            return ids
        except Exception as ex:
            raise SuperdeskApiError.internalError(str(ex), exception=ex)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import request
from eve.utils import config
from eve.methods.common import resolve_document_etag
import logging
import superdesk
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.utc import utcnow
from superdesk import get_backend
from superdesk import get_resource_service
from superdesk.workflow import get_privileged_actions
from superdesk.validation import ValidationError
from flask_babel import _


_preferences_key = 'preferences'
_user_preferences_key = 'user_preferences'
_session_preferences_key = 'session_preferences'
_privileges_key = 'active_privileges'
_action_key = 'allowed_actions'
logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = 'preferences'
    service = PreferencesService(endpoint_name, backend=get_backend())
    PreferencesResource(endpoint_name, app=app, service=service)
    app.on_session_end -= service.on_session_end
    app.on_session_end += service.on_session_end
    app.on_role_privileges_updated -= service.on_role_privileges_updated
    app.on_role_privileges_updated += service.on_role_privileges_updated
    superdesk.intrinsic_privilege(resource_name=endpoint_name, method=['PATCH'])


def enhance_document_with_default_prefs(doc):
    user_prefs = doc.get(_user_preferences_key, {})
    available = dict(superdesk.default_user_preferences)
    available.update(user_prefs)

    def sync_field(field, dest, default):
        if not isinstance(dest, dict) or not isinstance(default, dict):
            return
        if default.get(field):
            dest[field] = default[field]
        elif dest.get(field):
            dest.pop(field, None)

    # make sure label and category are up-to-date
    for k, v in available.items():
        default = superdesk.default_user_preferences.get(k)
        if default:
            sync_field('label', v, default)
            sync_field('category', v, default)

        if isinstance(v, dict) and v.get('label'):
            v['label'] = _(v['label'])
        if isinstance(v, dict) and v.get('category'):
            v['category_label'] = _(v['category'])

    doc[_user_preferences_key] = available


class PreferencesResource(Resource):
    datasource = {
        'source': 'users',
        'projection': {
            _session_preferences_key: 1,
            _user_preferences_key: 1,
            _privileges_key: 1,
            _action_key: 1,
            '_etag': 1,

            'role': 1,
            'user_type': 1,
            'privileges': 1,
        }
    }
    schema = {
        _session_preferences_key: {'type': 'dict', 'required': True, 'allow_unknown': True},
        _user_preferences_key: {'type': 'dict', 'required': True, 'allow_unknown': True},
        _privileges_key: {'type': 'dict', 'allow_unknown': True},
        _action_key: {'type': 'list'},

        # we need these to get user/role info
        'role': {'type': 'string'},
        'user_type': {'type': 'string'},
        'privileges': {'type': 'dict'},
    }
    allow_unknown = True
    resource_methods = []
    item_methods = ['GET', 'PATCH']
    merge_nested_documents = True

    superdesk.register_default_user_preference('feature:preview', {
        'type': 'bool',
        'enabled': False,
        'default': False,
        'label': _('Enable Feature Preview'),
        'category': _('feature'),
        'privileges': ['feature_preview']
    })

    superdesk.register_default_user_preference('archive:view', {
        'type': 'string',
        'allowed': ['mgrid', 'compact'],
        'view': 'mgrid',
        'default': 'mgrid',
        'label': _('Users archive view format'),
        'category': _('archive')
    })

    superdesk.register_default_user_preference('singleline:view', {
        'type': 'bool',
        'enabled': None,
        'default': False,
        'label': _('Enable Single Line View'),
        'category': _('rows')
    })

    superdesk.register_default_user_preference('editor:theme', {
        'type': 'string',
        'theme': '',
    })

    superdesk.register_default_user_preference('workqueue:items', {
        'items': []
    })

    superdesk.register_default_user_preference('dashboard:ingest', {
        'providers': []
    })

    superdesk.register_default_user_preference('agg:view', {
        'active': {},
    })

    superdesk.register_default_user_preference('templates:recent', {})

    superdesk.register_default_user_preference('dateline:located', {
        'type': 'dict',
        'label': _('Located'),
        'category': _('article_defaults')
    })

    superdesk.register_default_user_preference('categories:preferred', {
        'type': 'dict',
        'category': _('categories'),
        'label': _('Preferred Categories'),
        'selected': {},
    })

    superdesk.register_default_user_preference('desks:preferred', {
        'type': 'dict',
        'category': _('desks'),
        'label': _('Preferred Desks'),
        'selected': {},
    })

    superdesk.register_default_user_preference('article:default:place', {
        'type': 'list',
        'label': _('Place'),
        'category': _('article_defaults'),
        'place': []
    })

    superdesk.register_default_user_preference('spellchecker:status', {
        'type': 'bool',
        'enabled': True,
        'default': True
    })

    superdesk.register_default_user_preference('contacts:view', {
        'type': 'string',
        'allowed': ['mgrid', 'compact'],
        'view': 'mgrid',
        'default': 'mgrid',
        'label': _('Users contacts view format'),
        'category': _('contacts')
    })

    superdesk.register_default_user_preference('destination:active', {})

    superdesk.register_default_session_preference('scratchpad:items', [])
    superdesk.register_default_session_preference('desk:last_worked', '')
    superdesk.register_default_session_preference('desk:items', [])
    superdesk.register_default_session_preference('stage:items', [])
    superdesk.register_default_session_preference('pinned:items', [])


class PreferencesService(BaseService):

    def on_session_end(self, user_id, session_id):
        service = get_resource_service('users')
        user_doc = service.find_one(req=None, _id=user_id)
        session_prefs = user_doc.get(_session_preferences_key, {}).copy()

        if not isinstance(session_id, str):
            session_id = str(session_id)

        if session_id in session_prefs:
            del session_prefs[session_id]
            service.system_update(user_id, {_session_preferences_key: session_prefs}, user_doc)

    def set_session_based_prefs(self, session_id, user_id):
        service = get_resource_service('users')
        user_doc = service.find_one(req=None, _id=user_id)

        session_prefs = user_doc.get(_session_preferences_key, {})
        available = dict(superdesk.default_session_preferences)
        if available.get('desk:last_worked') == '' and user_doc.get('desk'):
            available['desk:last_worked'] = user_doc.get('desk')

        session_prefs.setdefault(str(session_id), available)
        service.system_update(user_id, {_session_preferences_key: session_prefs}, user_doc)

    def set_user_initial_prefs(self, user_doc):
        if _user_preferences_key not in user_doc:
            orig_user_prefs = user_doc.get(_preferences_key, {})
            available = dict(superdesk.default_user_preferences)
            available.update(orig_user_prefs)
            user_doc[_user_preferences_key] = available

    def find_one(self, req, **lookup):
        session = get_resource_service('sessions').find_one(req=None, _id=lookup['_id'])
        _id = session['user'] if session else lookup['_id']
        doc = get_resource_service('users').find_one(req, _id=_id)
        if doc:
            doc['_id'] = session['_id'] if session else _id
        return doc

    def on_fetched_item(self, doc):
        session_id = request.view_args['_id']
        session_prefs = doc.get(_session_preferences_key, {}).get(session_id, {})
        doc[_session_preferences_key] = session_prefs
        self.enhance_document_with_user_privileges(doc)
        enhance_document_with_default_prefs(doc)
        self._filter_preferences_by_privileges(doc)

    def on_update(self, updates, original):
        existing_user_preferences = original.get(_user_preferences_key, {}).copy()
        existing_session_preferences = original.get(_session_preferences_key, {}).copy()

        self.update_user_prefs(updates, existing_user_preferences)
        session_id = request.view_args['_id']
        self.update_session_prefs(updates, existing_session_preferences, session_id)

    def update_session_prefs(self, updates, existing_session_preferences, session_id):
        session_prefs = updates.get(_session_preferences_key)
        if session_prefs is not None:
            for k in (k for k, v in session_prefs.items() if k not in superdesk.default_session_preferences):
                raise ValidationError(_('Invalid preference: {preference}').format(preference=k))

            existing = existing_session_preferences.get(session_id, {})
            existing.update(session_prefs)
            existing_session_preferences[session_id] = existing
            updates[_session_preferences_key] = existing_session_preferences

    def update_user_prefs(self, updates, existing_user_preferences):
        user_prefs = updates.get(_user_preferences_key)
        if user_prefs is not None:
            # check if the input is validated against the default values
            for k in ((k for k, v in user_prefs.items() if k not in superdesk.default_user_preferences)):
                raise ValidationError(_('Invalid preference: {preference}').format(preference=k))

            existing_user_preferences.update(user_prefs)
            updates[_user_preferences_key] = existing_user_preferences

    def update(self, id, updates, original):
        session = get_resource_service('sessions').find_one(req=None, _id=original['_id'])
        original_unpatched = self.backend.find_one(self.datasource, req=None, _id=session['user'])
        updated = original_unpatched.copy()
        updated.update(updates)
        del updated['_id']
        res = self.backend.update(self.datasource, original_unpatched['_id'], updated, original_unpatched)
        updates.update(updated)
        # Return only the patched session prefs
        session_prefs = updates.get(_session_preferences_key, {}).get(str(original['_id']), {})
        updates[_session_preferences_key] = session_prefs
        self.enhance_document_with_user_privileges(updates)
        enhance_document_with_default_prefs(updates)
        return res

    def enhance_document_with_user_privileges(self, user_doc):
        role_doc = get_resource_service('users').get_role(user_doc)
        get_resource_service('users').set_privileges(user_doc, role_doc)
        user_doc[_action_key] = get_privileged_actions(user_doc[_privileges_key])

    def get_user_preference(self, user_id):
        """
        This function returns preferences for the user.
        """
        doc = get_resource_service('users').find_one(req=None, _id=user_id)
        prefs = doc.get(_user_preferences_key, {})
        return prefs

    def email_notification_is_enabled(self, user_id=None, preferences=None):
        """
        This function checks if email notification is enabled or not based on the preferences.
        """
        if user_id:
            preferences = self.get_user_preference(user_id)
        send_email = preferences.get('email:notification', {}) if isinstance(preferences, dict) else {}
        return send_email and send_email.get('enabled', False)

    def is_authorized(self, **kwargs):
        """
        Returns False if logged-in user is trying to update other user's or session's privileges.

        :param kwargs:
        :return: True if authorized, False otherwise
        """
        if not kwargs.get('_id') or not kwargs.get('user_id'):
            return False

        session = get_resource_service('sessions').find_one(req=None, _id=kwargs.get('_id'))
        if not session:
            return False

        return str(kwargs.get('user_id')) == str(session.get('user'))

    def on_role_privileges_updated(self, role, role_users):
        """Runs when user privilage has been updated.

        Update the session for active user so that preferences can be reloaded.

        :param dict role: role getting updated
        :param list role_users: list of user belonging to the role.
        """
        if not role_users or not role:
            return

        logger.info('On_Role_Privileges_Updated: Updating Users for Role:{}.'.format(role.get(config.ID_FIELD)))
        for user in role_users:
            try:
                super().update(user[config.ID_FIELD], {}, user)
            except Exception:
                logger.warn('On_Role_Privileges_Updated:Failed to update user:{} with role:{}.'.
                            format(user.get(config.ID_FIELD), role.get(config.ID_FIELD)), exc_info=True)

    def _filter_preferences_by_privileges(self, doc):
        privileges = doc[_privileges_key]
        preferences = doc[_user_preferences_key]

        def has_missing_privileges(prefs):
            prefs = prefs if isinstance(prefs, list) else [prefs]
            return [priv for pref in prefs for priv in pref.get('privileges', []) if not privileges.get(priv)]

        doc[_user_preferences_key] = {k: v for k, v in preferences.items() if not has_missing_privileges(v)}




from flask_babel import _

from .common import BasePublishResource, ITEM_UNPUBLISH, CONTENT_STATE, ITEM_KILL
from .kill import KillPublishService, PACKAGE_WORKFLOW


class UnpublishResource(BasePublishResource):
    def __init__(self, endpoint_name, app, service):
        super().__init__(endpoint_name, app, service, ITEM_UNPUBLISH)
    pass


class UnpublishService(KillPublishService):
    publish_type = ITEM_KILL
    item_operation = ITEM_UNPUBLISH
    published_state = CONTENT_STATE.UNPUBLISHED
    package_workflow = PACKAGE_WORKFLOW.IGNORE
    VALIDATE_ERROR_MESSAGE = _(
        'This item is in a package. It needs to be removed before the item can be unpublished.'
    )

    def __init__(self, datasource=None, backend=None):
        super().__init__(datasource=datasource, backend=backend)

    def apply_kill_override(self, item, updates):
        pass

    def apply_kill_template(self, item):
        pass

    def broadcast_kill_email(self, original, updates):
        pass

    def _publish_associated_items(self, original, updates=None):
        # associated items remain untouched during unpublish
        pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.resource import Resource, not_analyzed
from superdesk.services import BaseService
from superdesk.media.renditions import generate_renditions, get_renditions_spec
from superdesk import get_resource_service
from superdesk import errors
from flask import current_app
from PIL import Image, ImageEnhance
from io import BytesIO
from eve.utils import config
import os.path
import uuid
import logging

logger = logging.getLogger(__name__)


class MediaEditorResource(Resource):
    schema = {
        'item_id': {
            'type': 'string',
        },
        'item': {
            'type': 'dict',
        },
        'edit': {
            'type': 'dict',
            'required': True,
        },
        'renditions': {
            'type': 'dict',
        }
    }
    internal_resource = False
    resource_methods = ['POST']
    item_methods = []
    privileges = {'POST': 'archive'}


class MediaEditorService(BaseService):
    """Service givin metadata on backend itself"""

    def transform(self, im, operation, param):
        """Apply image transformation

        :param Image im: image to transform
        :param str operation: name of the operation to do
        :param param: parameters of the operation
        :return Image: resulting image
        """
        if operation == 'rotate':
            return im.rotate(int(param), expand=1)

        elif operation == 'flip':
            if param in ('vertical', 'both'):
                im = im.transpose(Image.FLIP_TOP_BOTTOM)
            if param in ('horizontal', 'both'):
                im = im.transpose(Image.FLIP_LEFT_RIGHT)
            return im

        elif operation == 'brightness':
            return ImageEnhance.Brightness(im).enhance(float(param))

        elif operation == 'contrast':
            return ImageEnhance.Contrast(im).enhance(float(param))

        elif operation == 'grayscale':
            return im.convert('L')

        elif operation == 'saturation':
            return ImageEnhance.Color(im).enhance(float(param))

        logger.warning('unhandled operation: {operation} {param}'.format(
            operation=operation,
            param=param))

        return im

    def create(self, docs):
        """Apply transformation requested in 'edit'"""
        ids = []
        archive = get_resource_service('archive')
        for doc in docs:
            # first we get item and requested edit operations
            item = doc.pop('item', None)
            if item is None:
                try:
                    item_id = doc.pop('item_id')
                except KeyError:
                    raise errors.SuperdeskApiError.badRequestError('either item or item_id must be specified')
            else:
                item_id = item[config.ID_FIELD]

            if item is None and item_id:
                item = next(archive.find({'_id': item_id}))
            edit = doc.pop('edit')

            # now we retrieve and load current original media
            rendition = item['renditions']['original']
            media_id = rendition['media']
            media = current_app.media.get(media_id)
            out = im = Image.open(media)

            # we apply all requested operations on original media
            for operation, param in edit.items():
                try:
                    out = self.transform(out, operation, param)
                except ValueError:
                    # if the operation can't be applied just ignore it
                    logger.warning('failed to apply operation: {operation} {param} for media {id}'.format(
                        operation=operation,
                        param=param,
                        id=media_id))
            buf = BytesIO()
            out.save(buf, format=im.format)

            # we set metadata
            buf.seek(0)
            content_type = rendition['mimetype']
            ext = os.path.splitext(rendition['href'])[1]
            filename = str(uuid.uuid4()) + ext

            # and save transformed media in database
            media_id = current_app.media.put(buf, filename=filename, content_type=content_type)

            # now we recreate other renditions based on transformed original media
            buf.seek(0)
            renditions = generate_renditions(buf,
                                             media_id,
                                             [],
                                             'image',
                                             content_type,
                                             get_renditions_spec(),
                                             current_app.media.url_for_media)

            ids.append(item_id)
            doc['renditions'] = renditions

        return [ids]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .iptc7901 import IPTC7901FeedParser
from superdesk.io.registry import register_feed_parser
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE


class DPAIPTC7901FeedParser(IPTC7901FeedParser):
    NAME = 'dpa_iptc7901'

    label = 'DPA IPTC 7901 Parser'

    def parse(self, file_path, provider=None):
        item = super().parse(file_path, provider)
        item = self.dpa_derive_dateline(item)
        self.dpa_parse_header(item)
        # Markup the text and set the content type
        item['body_html'] = '<p>' + item['body_html'].replace('\r\n', ' ').replace('\n', '</p><p>') + '</p>'
        item[ITEM_TYPE] = CONTENT_TYPE.TEXT
        return item

    def dpa_parse_header(self, item):
        """
        Try to pickout the headline, byline and take key from what is seemingly a header
        :param item:
        :return:
        """
        headers, divider, the_rest = item.get('body_html', '').partition(' =\r\n\n')
        # If no divider then there was only one line and that is the headline so clean up the stray '='
        if not divider:
            item['headline'] = item.get('headline').replace(' =', '')
            return

        headerlines = headers.split('\n')

        # If the last one is a byline, the line before is the headline
        if headerlines[-1].startswith('By '):
            item['byline'] = headerlines[-1].replace('By ', '')
            if len(headerlines) > 1:
                item['anpa_take_key'] = item.get('headline') if item.get('anpa_take_key') is None else item.get(
                    'anpa_take_key') + ' ' + item.get('headline')
                item['headline'] = headerlines[-2]
            if len(headerlines) > 2:
                item['anpa_take_key'] = headerlines[-3] if item.get('anpa_take_key') is None else item.get(
                    'anpa_take_key') + ' ' + headerlines[-3]
            item['body_html'] = the_rest
            return

        # Only a headline
        if len(headerlines) == 1:
            item['anpa_take_key'] = item.get('headline') if item.get('anpa_take_key') is None \
                else item.get('anpa_take_key') + ' ' + item.get('headline')
            item['headline'] = headerlines[-1]
            item['body_html'] = the_rest
            return

        # Take the headline as the last one and any preceding into the take key
        item['headline'] = headerlines[-1]
        if len(headerlines) > 1:
            item['anpa_take_key'] = headerlines[-2] if item.get('anpa_take_key') is None \
                else item.get('anpa_take_key') + ' ' + headerlines[-2]
        item['body_html'] = the_rest

    def dpa_derive_dateline(self, item):
        """Parse dateline from item body.

        This function attempts to parse a dateline from the first few lines of
        the item body and populate the dataline location, it also populates the dateline source.
        If a dateline is matched the coresponding string is removed from the article text.

        :param item:
        :return:
        """
        lines = item['body_html'].splitlines()
        if lines:
            # expect the dateline in the first 5 lines, sometimes there is what appears to be a headline preceeding it.
            for line_num in range(0, min(len(lines), 5)):
                city, source, the_rest = lines[line_num].partition(' (dpa) - ')
                # test if we found a candidate and ensure that the city starts the line and is not crazy long
                if source and lines[line_num].find(city) == 0 and len(city.strip()) < 20:
                    self.set_dateline(item, city.strip())
                    item['dateline']['source'] = 'dpa'
                    item['dateline']['text'] = city.strip()
                    item['body_html'] = item['body_html'].replace(city + source, '', 1)
                    break
        return item


register_feed_parser(DPAIPTC7901FeedParser.NAME, DPAIPTC7901FeedParser())



# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from . import oauth2
from .clients import AuthServerClientsResource, AuthServerClientsService


def init_app(app):
    oauth2.config_oauth(app)
    superdesk.register_resource('auth_server_clients', AuthServerClientsResource, AuthServerClientsService)
    superdesk.blueprint(oauth2.bp, app)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""This module contains common tools for spellchecker tests"""

from unittest.mock import MagicMock
from superdesk import get_resource_service


def mock_dictionaries(service, model):
    """A side effect to return a mocked dictionaries service

    other services are returned normally
    """
    if service == 'dictionaries':
        fake_service = MagicMock()
        fake_service.get_model_for_lang.return_value = model
        return fake_service
    else:
        return get_resource_service(service)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from superdesk.resource import Resource


class ContentFilterResource(Resource):
    schema = {
        'name': {
            'type': 'string',
            'required': True,
            'nullable': False,
            'empty': False,
            'iunique': True
        },
        'content_filter': {
            'type': 'list',
            'schema': {
                'type': 'dict',
                'schema': {
                    'expression': {
                        'type': 'dict',
                        'schema': {
                            'fc': {
                                'type': 'list',
                                'schema': Resource.rel('filter_conditions', True)
                            },
                            'pf': {
                                'type': 'list',
                                'schema': Resource.rel('content_filters', True)
                            }
                        }
                    }
                }
            }
        },
        'is_global': {
            'type': 'boolean',
            'default': False
        },
        'is_archived_filter': {
            'type': 'boolean',
            'default': False
        },
        'api_block': {
            'type': 'boolean',
            'default': False
        }
    }

    additional_lookup = {
        'url': 'regex("[\w,.:-]+")',
        'field': 'name'
    }

    privileges = {'POST': 'content_filters',
                  'PATCH': 'content_filters',
                  'DELETE': 'content_filters'}

    mongo_indexes = {
        'name_1': ([('name', 1)], {'unique': True}),
    }



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from bson import ObjectId
from unittest import mock
from unittest.mock import MagicMock
from datetime import timedelta
from collections import UserList

from superdesk.tests import TestCase
from superdesk.utc import utcnow
from apps.publish.enqueue import enqueue_service
from superdesk.publish.publish_queue import PUBLISHED_IN_PACKAGE
from superdesk.publish import publish_queue
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE


class QueueItemsTestCase(TestCase):
    """Tests for the get_queue_items() function."""

    def setUp(self):
        try:
            from superdesk.publish.publish_content import get_queue_items
        except ImportError:
            self.fail("Could not import function under test (transmit_items).")
        else:
            self.func_under_test = get_queue_items
            self.queue_items = [
                {'_id': ObjectId(), 'state': 'pending', 'item_id': 'item_1', 'item_version': 4,
                 'headline': 'pending headline', 'destination': {}},
                {'_id': ObjectId(), 'state': 'retrying', 'item_id': 'item_2', 'item_version': 4,
                 'headline': 'retrying headline', 'retry_attempt': 2,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=30)},
                {'_id': ObjectId(), 'state': 'success', 'item_id': 'item_3', 'item_version': 4,
                 'headline': 'success headline', 'retry_attempt': 4,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
                {'_id': ObjectId(), 'state': 'failed', 'item_id': 'item_4', 'item_version': 4,
                 'headline': 'failed headline', 'retry_attempt': 10,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
                {'_id': ObjectId(), 'state': 'canceled', 'item_id': 'item_5', 'item_version': 4,
                 'headline': 'canceled headline', 'retry_attempt': 4,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
                {'_id': ObjectId(), 'state': 'retrying', 'item_id': 'item_6', 'item_version': 4,
                 'headline': 'retrying headline', 'retry_attempt': 2,
                 'next_retry_attempt_at': utcnow() + timedelta(minutes=-30)},
            ]
            self.app.data.insert('publish_queue', self.queue_items)

    def test_get_queue_items(self):
        items = list(self.func_under_test())
        self.assertEqual(len(items), 1)
        for item in items:
            self.assertIn(item['item_id'], ['item_1'])

    def test_get_retry_queue_items(self):
        items = list(self.func_under_test(True))
        self.assertEqual(len(items), 1)
        for item in items:
            self.assertIn(item['item_id'], ['item_6'])

    def test_get_queue_items_with_retrying_items(self):
        item = self.app.data.find_one('publish_queue', req=None, _id=self.queue_items[1]['_id'])
        self.app.data.update('publish_queue', item.get('_id'),
                             {'next_retry_attempt_at': utcnow() - timedelta(minutes=30)},
                             item)
        items = list(self.func_under_test(True))
        self.assertEqual(len(items), 2)
        self.assertListEqual([item_l['item_id'] for item_l in items], ['item_2', 'item_6'])

    @mock.patch.object(enqueue_service, 'ObjectId')
    @mock.patch.object(enqueue_service, 'get_utc_schedule')
    @mock.patch.object(enqueue_service, 'get_resource_service')
    @mock.patch.object(enqueue_service, 'get_formatter')
    def test_enqueue_dict(self, *mocks):
        get_formatter, get_resource_service, _, _ = mocks
        publish_queue = get_resource_service.return_value
        fake_post = publish_queue.post
        service = enqueue_service.EnqueueService()
        fake_formatter = get_formatter.return_value
        doc_dict = {ITEM_TYPE: CONTENT_TYPE.TEXT, PUBLISHED_IN_PACKAGE: False}
        fake_doc = MagicMock()
        fake_doc.__getitem__ = lambda s, k: doc_dict.get(k, MagicMock())
        fake_doc.get = doc_dict.get
        fake_destination = MagicMock()
        fake_subscriber = MagicMock()
        subs_dict = {'destinations': [fake_destination], 'api_enabled': False}
        fake_subscriber.__getitem__ = lambda s, k: subs_dict.get(k, MagicMock())
        fake_subscriber['destinations'] = [fake_destination]
        subscribers = [fake_subscriber]
        fake_formatter.format.return_value = [{'published_seq_num': 42,
                                               'formatted_item': 'test OK'}]
        service.get_destinations = MagicMock(return_value=fake_subscriber['destinations'])
        service.queue_transmission(fake_doc, subscribers)
        self.assertEqual(len(fake_post.call_args_list), 1)
        self.assertEqual(len(fake_post.call_args_list[0][0][0]), 1)
        doc = fake_post.call_args_list[0][0][0][0]
        self.assertEqual(doc['published_seq_num'], 42)
        self.assertEqual(doc['formatted_item'], 'test OK')

        fake_post = publish_queue.post = MagicMock()
        fake_formatter.format.return_value = [{'this_should_not': 'work',
                                               'bad_key': 'value'}]
        service.queue_transmission(fake_doc, subscribers)
        # post should not have been called here,
        # because the dict is lacking the mandatory keys
        self.assertFalse(fake_post.called)

        fake_post = publish_queue.post = MagicMock()
        fake_formatter.format.return_value = [(42, 'test tuple OK')]
        service.queue_transmission(fake_doc, subscribers)
        self.assertEqual(len(fake_post.call_args_list), 1)
        self.assertEqual(len(fake_post.call_args_list[0][0][0]), 1)
        doc = fake_post.call_args_list[0][0][0][0]
        self.assertEqual(doc['published_seq_num'], 42)
        self.assertEqual(doc['formatted_item'], 'test tuple OK')

        fake_post = publish_queue.post = MagicMock()
        fake_formatter.format.return_value = [(1, "2", 3)]
        service.queue_transmission(fake_doc, subscribers)
        # post should not have been called here,
        # because the tuple should be in (published_seq_num, formatted_item) format
        self.assertFalse(fake_post.called)

    @mock.patch.object(publish_queue, 'app')
    def test_delete_encoded_item(self, fake_app):
        fake_storage = fake_app.storage
        fake_storage_delete = fake_storage.delete
        service = publish_queue.PublishQueueService(backend=MagicMock())
        service.get_from_mongo = MagicMock()
        cursor = UserList([{'_id': "4567", 'encoded_item_id': 'TEST ID'}])
        cursor.sort = MagicMock()
        cursor.sort.return_value = cursor
        service.get_from_mongo.return_value = cursor
        service.delete({'_id': "4567"})
        assert fake_storage_delete.call_args == mock.call('TEST ID')



import json
from flask import url_for

from superdesk import get_resource_service
from apps.archive.common import ARCHIVE


def test_service_get(prodapi_app_with_data):
    """
    Test fetching items using `archive` service
    :param prodapi_app_with_data: prod api app with filled data
    """

    with prodapi_app_with_data.app_context():
        items_service = get_resource_service(ARCHIVE)

        assert len(list(items_service.get(req=None, lookup={}))) == 7


def test_readonly(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Ensure that `archive` endpoint is readonly
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    with prodapi_app_with_data.test_request_context():
        for method, status in (('get', 200), ('post', 405), ('patch', 405), ('put', 405), ('delete', 405)):
            # we send a request
            resp = getattr(prodapi_app_with_data_client, method)(
                url_for('archive|resource')
            )
            # we get a response
            assert resp.status_code == status


def test_excluded_fields(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Ensure that fields which are listed as `excluded_fields` are not in the response.
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    excluded_fields = {
        'fields_meta',
        'unique_id',
        'family_id',
        'event_id',
        'lock_session',
        'lock_action',
        'lock_time',
        'lock_user',
        '_etag',
        '_type',
        '_updated',
        '_created',
        '_current_version',
        '_links',
    }

    with prodapi_app_with_data.test_request_context():
        # list
        resp = prodapi_app_with_data_client.get(url_for('archive|resource'))
        resp_data = json.loads(resp.data.decode('utf-8'))

        for item in resp_data['_items']:
            assert len(set(item.keys()) & excluded_fields) == 0

        # details
        item = resp_data['_items'][0]
        resp = prodapi_app_with_data_client.get(
            url_for('archive|item_lookup', _id=item['guid']),
        )
        resp_data = json.loads(resp.data.decode('utf-8'))

        assert len(set(resp_data.keys()) & excluded_fields) == 0


def test_renditions(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Ensure that rendition links are correct.
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    with prodapi_app_with_data.test_request_context():
        # details
        resp = prodapi_app_with_data_client.get(
            url_for(
                'archive|item_lookup',
                _id='urn:newsml:localhost:5000:2019-08-14T15:02:48.032188:d1be79f4-1d08-464a-90ac-90542dde4e90'
            ),
        )
        resp_data = json.loads(resp.data.decode('utf-8'))

        assert resp_data['associations']['editor_0']['renditions'] == {
            'baseImage': {
                'height': 933,
                'href': 'http://localhost:5500/prodapi/v1/assets/5d553c343031e2855a2e5664.jpg',
                'mimetype': 'image/jpeg',
                'poi': {
                    'x': 700,
                    'y': 466
                },
                'width': 1400
            },
            'original': {
                'height': 1280,
                'href': 'http://localhost:5500/prodapi/v1/assets/5d553c333031e2855a2e5662.jpg',
                'mimetype': 'image/jpeg',
                'poi': {
                    'x': 960,
                    'y': 640
                },
                'width': 1920
            },
            'thumbnail': {
                'height': 120,
                'href': 'http://localhost:5500/prodapi/v1/assets/5d553c343031e2855a2e5666.jpg',
                'mimetype': 'image/jpeg',
                'poi': {
                    'x': 90,
                    'y': 60
                },
                'width': 180
            },
            'viewImage': {
                'height': 426,
                'href': 'http://localhost:5500/prodapi/v1/assets/5d553c343031e2855a2e5668.jpg',
                'mimetype': 'image/jpeg',
                'poi': {
                    'x': 320,
                    'y': 213
                },
                'width': 640
            }
        }


def test_filters(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Test filtering.
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    with prodapi_app_with_data.test_request_context():
        # specify desk using source
        params = {
            'source': json.dumps({
                "query": {
                    "filtered": {
                        "filter": {
                            "terms": {
                                "task.desk": ["7777777775ec40943b60e"]
                            }
                        }
                    }
                }
            })
        }
        resp = prodapi_app_with_data_client.get(url_for('archive|resource', **params))
        resp_data = json.loads(resp.data.decode('utf-8'))

        assert len(resp_data['_items']) == 1
        assert resp_data['_items'][0]['guid'] == 'tag:localhost:5000:2019:1dbebe89-e808-43f5-a27c-91545dac896f'



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from copy import deepcopy
from flask import request
from flask_babel import _
from eve.utils import config

from apps.archive.usage import update_refs
from apps.archive.archive import SOURCE as ARCHIVE
from apps.content import push_item_move_notification
from superdesk.metadata.utils import item_url
from apps.archive.common import insert_into_versions, fetch_item
from superdesk.metadata.item import INGEST_ID, INGEST_VERSION, FAMILY_ID, ITEM_STATE, \
    CONTENT_STATE, GUID_FIELD
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.resource import Resource, build_custom_hateoas
from superdesk.services import BaseService
from superdesk.workflow import is_workflow_state_transition_valid
from superdesk import get_resource_service
from superdesk.metadata.packages import RESIDREF, REFS, GROUPS
from superdesk.metadata.item import MEDIA_TYPES, ASSOCIATIONS, get_schema

custom_hateoas = {'self': {'title': 'Archive', 'href': '/archive/{_id}'}}


class FetchResource(Resource):
    endpoint_name = 'fetch'
    resource_title = endpoint_name

    schema = get_schema(True)
    schema.update({
        'desk': Resource.rel('desks', False, required=True),
        'stage': Resource.rel('stages', False, nullable=True),
        'macro': {'type': 'string'},
        '_links': {'type': 'dict'},
    })

    url = 'ingest/<{0}:id>/fetch'.format(item_url)

    resource_methods = ['POST']
    item_methods = []

    privileges = {'POST': 'fetch'}


class FetchService(BaseService):

    def create(self, docs, **kwargs):
        return self.fetch(docs, id=request.view_args.get('id'), **kwargs)

    def fetch(self, docs, id=None, **kwargs):
        id_of_fetched_items = []

        for doc in docs:
            id_of_item_to_be_fetched = doc.get(config.ID_FIELD) if id is None else id

            desk_id = doc.get('desk')
            stage_id = doc.get('stage')

            ingest_service = get_resource_service('ingest')
            ingest_doc = ingest_service.find_one(req=None, _id=id_of_item_to_be_fetched)

            if not ingest_doc:
                raise SuperdeskApiError.notFoundError(
                    _('Fail to found ingest item with _id: {id}').format(id=id_of_item_to_be_fetched))

            if not is_workflow_state_transition_valid('fetch_from_ingest', ingest_doc[ITEM_STATE]):
                raise InvalidStateTransitionError()

            if doc.get('macro'):  # there is a macro so transform it
                macro_kwargs = kwargs.get('macro_kwargs') or {}
                ingest_doc = get_resource_service('macros').execute_macro(
                    ingest_doc,
                    doc.get('macro'),
                    dest_desk_id=desk_id,
                    dest_stage_id=stage_id,
                    **macro_kwargs,
                )

            dest_doc = fetch_item(ingest_doc, desk_id, stage_id, state=doc.get(ITEM_STATE), target=doc.get('target'))

            id_of_fetched_items.append(dest_doc[config.ID_FIELD])
            ingest_service.patch(id_of_item_to_be_fetched, {'archived': dest_doc['versioncreated']})

            dest_doc[FAMILY_ID] = ingest_doc[config.ID_FIELD]
            dest_doc[INGEST_ID] = self.__strip_version_from_guid(ingest_doc[GUID_FIELD], ingest_doc.get('version'))
            dest_doc[INGEST_VERSION] = ingest_doc.get('version')

            self.__fetch_items_in_package(dest_doc, desk_id, stage_id,
                                          doc.get(ITEM_STATE, CONTENT_STATE.FETCHED))

            self.__fetch_associated_items(dest_doc, desk_id, stage_id, doc.get(ITEM_STATE, CONTENT_STATE.FETCHED))

            desk = get_resource_service('desks').find_one(req=None, _id=desk_id)
            if desk and desk.get('default_content_profile'):
                dest_doc.setdefault('profile', desk['default_content_profile'])

            if dest_doc.get('type', 'text') in MEDIA_TYPES:
                dest_doc['profile'] = None

            update_refs(dest_doc, {})

            get_resource_service(ARCHIVE).post([dest_doc])
            insert_into_versions(doc=dest_doc)
            build_custom_hateoas(custom_hateoas, dest_doc)
            superdesk.item_fetched.send(self, item=dest_doc, ingest_item=ingest_doc)
            doc.update(dest_doc)

        if kwargs.get('notify', True):
            ingest_doc.update({'task': dest_doc.get('task')})
            push_item_move_notification(ingest_doc, doc, 'item:fetch')

        return id_of_fetched_items

    def __strip_version_from_guid(self, guid, version):
        """
        Checks if guid contains the version for the ingested item and returns the guid without version
        """
        try:
            if not version:
                return guid

            if guid.endswith(':{}'.format(str(version))):
                return guid[:-1 * (len(str(version)) + 1)]
        except Exception:
            return guid

    def __fetch_associated_items(self, doc, desk, stage, state):
        """
        Fetches the associated items of a given document
        """
        for key, item in (doc.get(ASSOCIATIONS) or {}).items():
            if item is not None and item.get('state') == 'ingested':
                new_item = deepcopy(item)
                new_item['desk'] = desk
                new_item['stage'] = stage
                new_item['state'] = state
                new_ids = self.fetch([new_item], id=None, notify=False)
                item[config.ID_FIELD] = new_ids[0]

    def __fetch_items_in_package(self, dest_doc, desk, stage, state):
        # Note: macro and target information is not user for package publishing.
        # Needs to looked later when package ingest requirements is clear.
        for ref in [ref for group in dest_doc.get(GROUPS, [])
                    for ref in group.get(REFS, []) if ref.get(RESIDREF)]:
            ref['location'] = ARCHIVE

        refs = [{config.ID_FIELD: ref.get(RESIDREF), 'desk': desk,
                 'stage': stage, ITEM_STATE: state}
                for group in dest_doc.get(GROUPS, [])
                for ref in group.get(REFS, []) if ref.get(RESIDREF)]

        if refs:
            new_ref_guids = self.fetch(refs, id=None, notify=False)
            count = 0
            for ref in [ref for group in dest_doc.get(GROUPS, [])
                        for ref in group.get(REFS, []) if ref.get(RESIDREF)]:
                ref[RESIDREF] = ref[GUID_FIELD] = new_ref_guids[count]
                count += 1


superdesk.workflow_state(CONTENT_STATE.FETCHED)
superdesk.workflow_action(
    name='fetch_from_ingest',
    include_states=['ingested'],
    privileges=['ingest', 'archive', 'fetch']
)

superdesk.workflow_state('routed')
superdesk.workflow_action(
    name='route',
    include_states=['ingested']
)




import os
import tempfile

from PIL import Image
from superdesk.tests import TestCase
from superdesk.media.renditions import generate_renditions, get_renditions_spec

IMG_PATH = os.path.join(os.path.dirname(__file__), 'fixtures', 'iphone_gpsinfo_exif.JPG')
BIG_IMG_PATH = os.path.join(
    os.path.dirname(os.path.dirname(os.path.dirname(__file__))),
    'features', 'steps', 'fixtures', 'bike.jpg',
)


class GenerateRenditionsTestCase(TestCase):

    def setUp(self):
        self.app.data.insert('vocabularies', [
            {
                '_id': 'crop_sizes',
                'items': [
                    {'is_active': True, 'name': 'landscape', 'width': 200, 'height': 100},
                    {'is_active': True, 'name': 'portrait', 'width': 150, 'height': 200},
                ],
            },
        ])

    def save_binary(self, rendition):
        """Used while debugging to check the picture."""
        binary = self.app.media.get(rendition['media'])
        image = Image.open(binary)
        fd, path = tempfile.mkstemp('.jpg')
        with open(fd, 'w') as out:
            image.save(out, 'jpeg')
            print('saved', path)

    def test_generate_renditions_custom_crop(self):
        inserted = []
        renditions = get_renditions_spec()
        with open(IMG_PATH, 'rb') as original:
            generated = generate_renditions(original, 'id', inserted, 'image', 'image/jpeg', renditions,
                                            self.app.media.url_for_media)

        # thumbnail should keep 4:3 ration
        self.assertIn('thumbnail', generated)
        self.assertEqual(160, generated['thumbnail']['width'])
        self.assertEqual(120, generated['thumbnail']['height'])

        # landscape should be cropped to 200x100
        self.assertIn('landscape', generated)
        landscape = generated['landscape']
        self.assertEqual(200, landscape['width'])
        self.assertEqual(100, landscape['height'])
        self.assertEqual(0, landscape['CropLeft'])
        self.assertEqual(400, landscape['CropRight'])
        self.assertEqual(50, landscape['CropTop'])
        self.assertEqual(250, landscape['CropBottom'])

        # portrait should be cropped to 150x200
        self.assertIn('portrait', generated)
        portrait = generated['portrait']
        self.assertEqual(150, portrait['width'])
        self.assertEqual(200, portrait['height'])
        self.assertEqual(0, portrait['CropTop'])
        self.assertEqual(300, portrait['CropBottom'])
        self.assertEqual(87, portrait['CropLeft'])
        self.assertEqual(312, portrait['CropRight'])

    def test_generate_renditions_base_image(self):
        inserted = []
        renditions = get_renditions_spec()
        with open(BIG_IMG_PATH, 'rb') as original:
            generated = generate_renditions(original, 'id', inserted, 'image', 'image/jpeg', renditions,
                                            self.app.media.url_for_media)

        self.assertIn('landscape', generated)
        landscape = generated['landscape']
        self.assertEqual(200, landscape['width'])
        self.assertEqual(100, landscape['height'])
        self.assertEqual(0, landscape['CropLeft'])
        self.assertEqual(1200, landscape['CropRight'])
        self.assertEqual(500, landscape['CropTop'])
        self.assertEqual(1100, landscape['CropBottom'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from ..models.item_autosave import ItemAutosaveModel
from apps.common.components.base_component import BaseComponent
from apps.common.models.utils import get_model
from apps.item_lock.models.item import ItemModel
from superdesk.errors import SuperdeskApiError
from flask_babel import _


class ItemAutosave(BaseComponent):
    def __init__(self, app):
        self.app = app

    @classmethod
    def name(cls):
        return 'archive_autosave'

    def autosave(self, item_id, updates, user, etag):
        updates.setdefault('_type', 'archive')
        item_model = get_model(ItemModel)
        item = item_model.find_one({'_id': item_id})
        if item is None:
            raise SuperdeskApiError.notFoundError(_('Invalid item identifier'))

        lock_user = item.get('lock_user', None)
        if lock_user and str(lock_user) != str(user['_id']):
            raise SuperdeskApiError.forbiddenError(_('The item was locked by another user'))

        autosave_model = get_model(ItemAutosaveModel)
        item.update(updates)
        self.app.on_item_autosave(item)
        autosave_item = autosave_model.find_one({'_id': item_id})
        if not autosave_item:
            autosave_model.create([item])
        else:
            autosave_model.update({'_id': item_id}, item, etag)
        self.app.on_item_autosaved(item)
        updates.update(item)
        return updates

    def clear(self, item_id):
        autosave_model = get_model(ItemAutosaveModel)
        return autosave_model.delete({'_id': item_id})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re

from datetime import datetime

from superdesk.errors import IngestApiError, ParserError
from superdesk.io.registry import register_feeding_service
from superdesk.io.feeding_services.http_base_service import HTTPFeedingServiceBase

utcfromtimestamp = datetime.utcfromtimestamp


class BBCLDRSFeedingService(HTTPFeedingServiceBase):
    """
    Feeding Service class for reading BBC's Local Democracy Reporting Service
    """

    # Following the api spec at https://docs.ldrs.org.uk/

    NAME = 'bbc_ldrs'
    ERRORS = [ParserError.parseMessageError().get_error_description()]

    label = 'BBC Local Democracy Reporter Service'

    fields = [
        {
            'id': 'url', 'type': 'text', 'label': 'LDRS URL',
            'placeholder': 'LDRS URL', 'required': True,
            'default': 'https://api.ldrs.org.uk/v1/item'
        },
        {
            'id': 'api_key', 'type': 'text', 'label': 'API Key',
            'placeholder': 'API Key', 'required': True,
            'default': ''
        }
    ]
    HTTP_AUTH = False

    def __init__(self):
        super().__init__()

    def _test(self, provider):
        config = self.config
        url = config['url']
        api_key = config['api_key']

        # limit the data to a single article and filter out all article fields
        # to save bandwidth
        params = {'limit': 1, 'fields': 'id'}
        headers = {'apikey': api_key}

        self.get_url(url, params=params, headers=headers)

    def _update(self, provider, update):
        json_items = self._fetch_data()
        parsed_items = []

        for item in json_items:
            try:
                parser = self.get_feed_parser(provider, item)
                parsed_items.append(parser.parse(item))
            except Exception as ex:
                raise ParserError.parseMessageError(ex, provider, data=item)

        return parsed_items

    def _fetch_data(self):
        url = self.config['url']
        api_key = self.config['api_key']

        last_update = self.provider.get('last_updated', utcfromtimestamp(0)).strftime('%Y-%m-%dT%H:%M:%S')

        # Results are pagified so we'll read this many at a time
        offset_jump = 10

        params = {'start': last_update, 'limit': offset_jump}
        headers = {'apikey': api_key}

        items = []

        offset = 0
        while True:
            params['offset'] = offset

            response = self.get_url(url, params=params, headers=headers)
            # The total number of results are given to us in json, get them
            # via a regex to read the field so we don't have to convert the
            # whole thing to json pointlessly
            item_ident = re.search('\"total\": *[0-9]*', response.text).group()
            results_str = re.search('[0-9]+', item_ident).group()

            if results_str is None:
                raise IngestApiError.apiGeneralError(
                    Exception(response.text), self.provider)

            num_results = int(results_str)

            if num_results > 0:
                items.append(response.text)

            if offset >= num_results:
                return items

            offset += offset_jump

        return items


register_feeding_service(BBCLDRSFeedingService)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from bson import ObjectId
from unittest import mock
from unittest.mock import MagicMock

from celery.exceptions import SoftTimeLimitExceeded

import superdesk
from superdesk.tests import TestCase
from superdesk.publish.publish_queue import PUBLISHED_IN_PACKAGE


class SoftTimeLimitTestCase(TestCase):

    @mock.patch('superdesk.publish.registered_transmitters')
    def test_softtimeout_exception(self, *mocks):
        self.app.config['MAX_TRANSMIT_RETRY_ATTEMPT'] = 4
        self.app.config['CELERY_TASK_ALWAYS_EAGER'] = False
        subscriber = {
            '_id': ObjectId('56c11bd78b84bb00b0a1905e'),
            'sequence_num_settings': {'max': 9999, 'min': 1},
            '_etag': 'f16b7eaa566f68b8d2561c811ec694bdf819784d',
            'is_active': True,
            'destinations': [{'delivery_type': 'email'}],
            'email': 'test@test.com',
            'subscriber_type': 'digital',
            'name': 'Test',
        }

        self.app.data.insert('subscribers', [subscriber])

        items = [{'_id': ObjectId(), 'state': 'pending', 'item_id': 'item_1', 'item_version': 4,
                  'headline': 'pending headline', 'destination': {'delivery_type': 'email'},
                  'subscriber_id': subscriber['_id'], 'formatted_item': 'test'},
                 {'_id': ObjectId(), 'state': 'pending', 'item_id': 'item_2', 'item_version': 4,
                  'headline': 'pending headline 2', 'destination': {'delivery_type': 'email'},
                  'subscriber_id': subscriber['_id'], 'formatted_item': 'test'}
                 ]

        self.app.data.insert('publish_queue', items)

        fake_transmitter = MagicMock()
        fake_transmitter.transmit.side_effect = SoftTimeLimitExceeded()

        fake_transmitters_list = mocks[0]
        fake_transmitters_list.__getitem__.return_value = fake_transmitter

        with self.assertRaises(SoftTimeLimitExceeded):
            superdesk.publish.publish_content.transmit_subscriber_items(subscriber.get('_id'), False)
        failed_item = self.app.data.find_one('publish_queue', req=None, _id=items[0].get('_id'))
        self.assertEqual(failed_item['state'], 'retrying')
        pending_item = self.app.data.find_one('publish_queue', req=None, _id=items[1].get('_id'))
        self.assertEqual(pending_item['state'], 'pending')
        self.app.config['CELERY_TASK_ALWAYS_EAGER'] = True



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .length_yards_to_metric import yards_to_metric


class YardTestCase(unittest.TestCase):

    def test_y_to_m(self):
        text = 'Tom runs 125 yd in 11 seconds. ' \
               'Tom runs 125 yards in 11 seconds. ' \
               'Tom runs 125 Yards in 11 seconds. ' \
               'Tom runs 125-Yards in 11 seconds. ' \
               'Tom runs 12.5 yd in 11 seconds. ' \
               'Tom runs 12,500 yards in 11 seconds. ' \
               'Tom runs 100-12500 Yards in 11 seconds. ' \

        item = {'body_html': text}
        res, diff = yards_to_metric(item)
        self.assertEqual(diff['125 yd'], '125 yd (114 metres)')
        self.assertEqual(diff['125 yards'], '125 yards (114 metres)')
        self.assertEqual(diff['125 Yards'], '125 Yards (114 metres)')
        self.assertEqual(diff['125-Yards'], '125-Yards (114 metres)')
        self.assertEqual(diff['12.5 yd'], '12.5 yd (11.4 metres)')
        self.assertEqual(diff['12,500 yards'], '12,500 yards (11,430 metres)')
        self.assertEqual(diff['100-12500 Yards'], '100-12500 Yards (91-11,430 metres)')
        self.assertEqual(res['body_html'], item['body_html'])



from superdesk.resource import Resource
from superdesk.metadata.utils import item_url
from superdesk.auth_server.scopes import Scope


class UsersResource(Resource):
    url = 'users'
    item_url = item_url
    item_methods = ['GET']
    resource_methods = ['GET']
    allow_unknown = True
    datasource = {
        'source': 'users',
        'default_sort': [('username', 1)]
    }
    privileges = {
        'GET': Scope.USERS_READ.name
    }



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .weight_pounds_to_metric import pounds_to_metric


class VolumeTestCase(unittest.TestCase):

    def test(self):
        text = 'Total volume is 100.50 pounds for this land. '\
               'Total volume is 15.7 lbs. for this land. '\
               'Total volume is 1 Pound for this land. '\
               'Total volume is 1-16 lb for this land. '\
               'Total volume is 1-16 lbs for this land. '\
               'Total volume is 16.7-pound for this land. '\
               'Total volume is 16,500-lb. for this land. '\

        item = {'body_html': text}
        res, diff = pounds_to_metric(item)
        self.assertEqual(diff['100.50 pounds'], '100.50 pounds (45.59 kg)')
        self.assertEqual(diff['15.7 lbs'], '15.7 lbs (7.1 kg)')
        self.assertEqual(diff['1 Pound'], '1 Pound (0.45 kg)')
        self.assertEqual(diff['1-16 lb'], '1-16 lb (0.45-7 kg)')
        self.assertEqual(diff['1-16 lbs'], '1-16 lbs (0.45-7 kg)')
        self.assertEqual(diff['16.7-pound'], '16.7-pound (7.6 kg)')
        self.assertEqual(diff['16,500-lb'], '16,500-lb (7,484 kg)')
        self.assertEqual(res['body_html'], item['body_html'])



import os
import time

from datetime import timedelta
from unittest.mock import patch, Mock

from superdesk.utc import utcnow
from superdesk.tests import TestCase
from superdesk.storage import AmazonMediaStorage
from superdesk.media.media_operations import guess_media_extension


class AmazonMediaStorageTestCase(TestCase):

    def setUp(self):
        self.amazon = AmazonMediaStorage(self.app)

        # Patch config with defaults
        p = patch.dict(self.app.config, {
            'AMAZON_SECRET_ACCESS_KEY': None,
            'AMAZON_CONTAINER_NAME': 'acname',
            'AMAZON_REGION': 'us-east-1',
            'AMAZON_S3_SUBFOLDER': '',
            'MEDIA_PREFIX': 'https://acname.s3-us-east-1.amazonaws.com'
        })
        p.start()
        self.addCleanup(p.stop)

    def test_media_url(self):
        filename = 'test'
        # automatic version is set on 15mins granularity.
        mins_granularity = int(int(time.strftime('%M')) / 4) * 4
        time_id = '%s%s' % (time.strftime('%Y%m%d%H%m'), mins_granularity)
        media_id = self.amazon.media_id(filename)
        self.assertEqual('%s/%s' % (time_id, filename), media_id)
        self.assertEqual(
            self.amazon.url_for_media(media_id),
            'https://acname.s3-us-east-1.amazonaws.com/%s' % media_id
        )
        sub = 'test-sub'
        settings = {
            'AMAZON_S3_SUBFOLDER': sub,
            'MEDIA_PREFIX': 'https://acname.s3-us-east-1.amazonaws.com/' + sub
        }
        with patch.dict(self.app.config, settings):
            media_id = self.amazon.media_id(filename)
            self.assertEqual('%s/%s' % (time_id, filename), media_id)
            path = '%s/%s' % (sub, media_id)
            self.assertEqual(
                self.amazon.url_for_media(media_id),
                'https://acname.s3-us-east-1.amazonaws.com/%s' % path
            )
            with patch.object(self.amazon, 'client') as s3:
                self.amazon.get(media_id)
                self.assertTrue(s3.get_object.called)
                self.assertEqual(
                    s3.get_object.call_args[1],
                    dict(Bucket='acname', Key=path)
                )

    def test_put_and_delete(self):
        """Test amazon if configured.

        If the environment variables have a Amazon secret key set then assume
        that we can attempt to put and delete into s3

        :return:
        """
        if self.app.config['AMAZON_SECRET_ACCESS_KEY']:
            id = self.amazon.put('test', content_type='text/plain')
            self.assertIsNot(id, None)
            self.assertTrue(self.amazon.exists(id))
            fromS3 = self.amazon.get(id)
            self.assertEqual(fromS3.read().decode('utf-8'), 'test')
            self.amazon.delete(id)
            self.assertFalse(self.amazon.exists(id))
        else:
            self.assertTrue(True)

    def test_put_into_folder(self):
        data = b'test data'
        folder = 's3test'
        filename = 'abc123.zip'
        content_type = 'application/zip'
        self.amazon.client.put_object = Mock()
        self.amazon.media_id = Mock(return_value=filename)
        self.amazon._check_exists = Mock(return_value=False)

        self.amazon.put(data, filename, content_type, folder=folder)

        kwargs = {
            'Key': '{}/{}'.format(folder, filename),
            'Body': data,
            'Bucket': 'acname',
            'ContentType': content_type,
        }
        self.amazon.client.put_object.assert_called_once_with(**kwargs)

    def test_find_folder(self):
        self.amazon.client = Mock()

        # Mock getting list of files from Amazon, first request returns a file, second request returns empty list
        self.amazon.client.list_objects = Mock(side_effect=[
            {'Contents': [{
                'Key': 'gridtest/abcd1234',
                'LastModified': utcnow() - timedelta(minutes=30),
                'Size': 500,
                'ETag': 'abcd1234'
            }]},
            {'Contents': []}
        ])

        folder = 'gridtest'
        self.amazon.find(folder=folder)

        call_arg_list = [({
            'Bucket': 'acname',
            'Marker': '',
            'MaxKeys': 1000,
            'Prefix': '{}/'.format(folder)
        },), ({
            'Bucket': 'acname',
            'Marker': 'gridtest/abcd1234',
            'MaxKeys': 1000,
            'Prefix': '{}/'.format(folder)
        },)]

        # We test the call_args_list as self.amazon.client.list_objects would have been called twice
        self.assertEqual(self.amazon.client.list_objects.call_count, 2)
        self.assertEqual(self.amazon.client.list_objects.call_args_list, call_arg_list)

    def test_guess_extension(self):
        self.assertEqual('.jpg', guess_media_extension('image/jpeg'))
        self.assertEqual('.png', guess_media_extension('image/png'))

        self.assertEqual('.mp3', guess_media_extension('audio/mp3'))
        self.assertEqual('.mp3', guess_media_extension('audio/mpeg'))
        self.assertEqual('.flac', guess_media_extension('audio/flac'))

        self.assertEqual('.mp4', guess_media_extension('video/mp4'))

        # leave empty when there is no extension
        self.assertEqual('', guess_media_extension('audio/foo'))

    def test_media_url_none_utf8(self):
        filename = '[DIARY NOTE] – Victory In The Pacific Day Commemoration - Thursday (1)'
        # automatic version is set on 15mins granularity.
        mins_granularity = int(int(time.strftime('%M')) / 4) * 4
        time_id = '%s%s' % (time.strftime('%Y%m%d%H%m'), mins_granularity)
        media_id = self.amazon.media_id(filename)
        self.assertEqual('%s/%s' % (time_id, 'DIARY NOTE - Victory In The Pacific Day Commemoration - Thursday (1)'),
                         media_id)
        self.assertEqual(
            self.amazon.url_for_media(media_id),
            'https://acname.s3-us-east-1.amazonaws.com/%s' % media_id
        )
        sub = 'test-sub'
        settings = {
            'AMAZON_S3_SUBFOLDER': sub,
            'MEDIA_PREFIX': 'https://acname.s3-us-east-1.amazonaws.com/' + sub
        }
        with patch.dict(self.app.config, settings):
            media_id = self.amazon.media_id(filename)
            self.assertEqual(
                '%s/%s' % (time_id, 'DIARY NOTE - Victory In The Pacific Day Commemoration - Thursday (1)'), media_id)
            path = '%s/%s' % (sub, media_id)
            self.assertEqual(
                self.amazon.url_for_media(media_id),
                'https://acname.s3-us-east-1.amazonaws.com/%s' % path
            )
            with patch.object(self.amazon, 'client') as s3:
                self.amazon.get(media_id)
                self.assertTrue(s3.get_object.called)
                self.assertEqual(
                    s3.get_object.call_args[1],
                    dict(Bucket='acname', Key=path)
                )

    def test_put_into_folder_none_utf8(self):
        data = b'test data'
        folder = 's3test'
        filename = '[DIARY NOTE] – Victory In The Pacific Day Commemoration - Thursday (1).pdf'
        content_type = 'application/pdf'
        self.amazon.client.put_object = Mock()
        self.amazon._check_exists = Mock(return_value=False)

        self.amazon.put(data, filename, content_type, folder=folder, version=False)

        kwargs = {
            'Key': '{}/{}'.format(folder, 'DIARY NOTE - Victory In The Pacific Day Commemoration - Thursday (1).pdf'),
            'Body': data,
            'Bucket': 'acname',
            'ContentType': content_type,
        }
        self.amazon.client.put_object.assert_called_once_with(**kwargs)

    def test_get_none_utf8(self):
        self.amazon.client.get_object = Mock()
        self.amazon.extract_metadata_from_headers = Mock(return_value={})

        self.amazon.get('[DIARY NOTE] – Victory In The Pacific Day Commemoration - Thursday (1).pdf')

        kwargs = {
            'Bucket': 'acname',
            'Key': 'DIARY NOTE - Victory In The Pacific Day Commemoration - Thursday (1).pdf'
        }
        self.amazon.client.get_object.assert_called_once_with(**kwargs)

    def test_mimetype_detect(self):
        # keep default mimetype
        content = b'bytes are here'
        filename = 'extensionless'
        content_type = 'text/css'
        folder = 'f1'
        self.amazon.client.put_object = Mock()
        self.amazon._check_exists = Mock(return_value=False)
        self.amazon.media_id = Mock(return_value=filename)
        self.amazon.put(content, filename, content_type, folder=folder)
        self.amazon.client.put_object.assert_called_once_with(
            **{
                'Key': '{}/{}'.format(folder, filename),
                'Body': content,
                'Bucket': 'acname',
                'ContentType': content_type
            }
        )

        # get mimetype from the filename
        content = b'bytes are here'
        filename = 'styles.css'
        content_type = 'application/pdf'
        folder = 'f1'
        self.amazon.client.put_object = Mock()
        self.amazon._check_exists = Mock(return_value=False)
        self.amazon.media_id = Mock(return_value=filename)
        self.amazon.put(content, filename, content_type, folder=folder)
        self.amazon.client.put_object.assert_called_once_with(
            **{
                'Key': '{}/{}'.format(folder, filename),
                'Body': b'bytes are here',
                'Bucket': 'acname',
                'ContentType': 'text/css'
            }
        )

        content = b'bytes are here'
        filename = 'styles.JpG'
        content_type = 'application/pdf'
        folder = 'f1'
        self.amazon.client.put_object = Mock()
        self.amazon._check_exists = Mock(return_value=False)
        self.amazon.media_id = Mock(return_value=filename)
        self.amazon.put(content, filename, content_type, folder=folder)
        self.amazon.client.put_object.assert_called_once_with(
            **{
                'Key': '{}/{}'.format(folder, filename),
                'Body': b'bytes are here',
                'Bucket': 'acname',
                'ContentType': 'image/jpeg'
            }
        )

        # get mimetype from the file
        fixtures_path = os.path.join(os.path.dirname(__file__), 'fixtures')

        with open(os.path.join(fixtures_path, "file_example-jpg.jpg"), 'rb') as content:
            filename = 'extensionless'
            content_type = 'dummy/text'
            folder = 'f1'
            self.amazon.client.put_object = Mock()
            self.amazon._check_exists = Mock(return_value=False)
            self.amazon.media_id = Mock(return_value=filename)
            self.amazon.put(content, filename, content_type, folder=folder)
            self.amazon.client.put_object.assert_called_once_with(
                **{
                    'Key': '{}/{}'.format(folder, filename),
                    'Body': content,
                    'Bucket': 'acname',
                    'ContentType': 'image/jpeg'
                }
            )

            with open(os.path.join(fixtures_path, "file_example-xls.xls"), 'rb') as content:
                filename = 'extensionless'
                content_type = 'dummy/text'
                folder = 'f1'
                self.amazon.client.put_object = Mock()
                self.amazon._check_exists = Mock(return_value=False)
                self.amazon.media_id = Mock(return_value=filename)
                self.amazon.put(content, filename, content_type, folder=folder)
                self.amazon.client.put_object.assert_called_once_with(
                    **{
                        'Key': '{}/{}'.format(folder, filename),
                        'Body': content,
                        'Bucket': 'acname',
                        'ContentType': 'application/vnd.ms-excel'
                    }
                )

            with open(os.path.join(fixtures_path, "file_example-xlsx.xlsx"), 'rb') as content:
                filename = 'extensionless'
                content_type = 'dummy/text'
                folder = 'f1'
                self.amazon.client.put_object = Mock()
                self.amazon._check_exists = Mock(return_value=False)
                self.amazon.media_id = Mock(return_value=filename)
                self.amazon.put(content, filename, content_type, folder=folder)
                self.amazon.client.put_object.assert_called_once_with(
                    **{
                        'Key': '{}/{}'.format(folder, filename),
                        'Body': content,
                        'Bucket': 'acname',
                        'ContentType': 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
                    }
                )

            with open(os.path.join(fixtures_path, "file_example-docx.docx"), 'rb') as content:
                filename = 'extensionless'
                content_type = 'dummy/text'
                folder = 'f1'
                self.amazon.client.put_object = Mock()
                self.amazon._check_exists = Mock(return_value=False)
                self.amazon.media_id = Mock(return_value=filename)
                self.amazon.put(content, filename, content_type, folder=folder)
                self.amazon.client.put_object.assert_called_once_with(
                    **{
                        'Key': '{}/{}'.format(folder, filename),
                        'Body': content,
                        'Bucket': 'acname',
                        'ContentType': 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
                    }
                )



import logging
from flask import current_app as app
from flask_babel import lazy_gettext
from superdesk import get_resource_service

logger = logging.getLogger(__name__)


def get_default_content_template(item, **kwargs):
    if 'dest_desk_id' in kwargs:
        desk = None
        desk_id = kwargs['dest_desk_id']
    elif 'desk' in kwargs:
        desk = kwargs['desk']
        desk_id = kwargs['desk']['_id']
    elif 'task' in item and 'desk' in item['task']:
        desk = None
        desk_id = item['task'].get('desk')
    else:
        logger.warning("Can't set default data, no desk identifier found")
        return

    if desk is None:
        desk = get_resource_service('desks').find_one(req=None, _id=desk_id)
    if not desk:
        logger.warning('Can\'t find desk with id "{desk_id}"'.format(desk_id=desk_id))
        return

    content_template_id = desk.get("default_content_template")
    if not content_template_id:
        logger.warning("No default content template set for {desk_name}".format(
            desk_name=desk.get('name', desk_id)))
        return
    content_template = get_resource_service("content_templates").find_one(req=None, _id=content_template_id)
    if not content_template:
        logger.warning('Can\'t find content_template with id "{content_template_id}"'.format(
            content_template_id=content_template_id))
        return

    return content_template


def set_default_template_metadata(item, **kwargs):

    fields_to_exclude = app.config.get('DEFAULT_TEMPLATE_METADATA_MACRO_EXCLUDE', {})
    fields_to_override = app.config.get('DEFAULT_TEMPLATE_METADATA_MACRO_OVERRIDE', {})

    """Replace some metadata from default content template"""

    content_template = get_default_content_template(item, **kwargs)
    if not content_template:
        return

    data = content_template['data']

    vocabularies = get_resource_service('vocabularies').get(req=None, lookup={'field_type': {'$exists': True}})
    for vocabulary in vocabularies:
        fields_to_exclude.append(vocabulary['_id'])

    for key, value in data.items():
        if (not item.get(key) and key not in fields_to_exclude) or key in fields_to_override:
            item[key] = data.get(key)

    # subject contains remaining metadata to copy
    subject = data.setdefault('subject', [])

    # we first take out the metadata which we want to add, if any
    to_add = []
    for sub in subject:
        if (sub.get('scheme')
                and not any(subjects.get('scheme') == sub.get('scheme') for subjects in item.get('subject', []))):
            to_add.append(sub)

    # and now we add the new one
    item.setdefault('subject', []).extend([i for i in to_add if i.get('scheme') and to_add])

    if not item['subject'] and data.get('subject'):
        item['subject'] = data.get('subject')

    return item


name = 'Set Default Template Metadata'
label = lazy_gettext('Set Default Template Metadata')
callback = set_default_template_metadata
access_type = 'backend'
action_type = 'direct'



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import logging
import superdesk

from io import BytesIO
from flask import current_app as app

from superdesk.ftp import ftp_connect
from superdesk.publish import register_transmitter
from superdesk.publish.publish_service import get_publish_service, PublishService
from superdesk.errors import PublishFtpError
from superdesk.media.renditions import get_rendition_file_name, get_renditions_spec

errors = [PublishFtpError.ftpError().get_error_description()]

logger = logging.getLogger(__name__)

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse


def get_renditions_filter():
    renditions = set(get_renditions_spec(
        without_internal_renditions=True
    ).keys())
    renditions.add('original')
    return renditions


class FTPPublishService(PublishService):
    """FTP Publish Service.

    It creates files on configured FTP server.

    :param string username: auth username
    :param string password: auth password
    :param path: server path
    :param passive: use passive mode (on by default)
    """

    NAME = 'FTP'
    CONFIG = {'passive': True}

    def config_from_url(self, url):
        """Parse given url into ftp config. Used for tests.

        :param url: url in form `ftp://username:password@host:port/dir`
        """
        url_parts = urlparse(url)
        return {
            'username': url_parts.username,
            'password': url_parts.password,
            'host': url_parts.hostname,
            'path': url_parts.path.lstrip('/'),
        }

    def _get_published_item(self, queue_item):
        try:
            return json.loads(queue_item['formatted_item'])
        except json.JSONDecodeError as ex:
            return superdesk.get_resource_service('published').find_one(
                req=None,
                item_id=queue_item['item_id'],
                _current_version=queue_item['item_version'],
            )

    def _transmit(self, queue_item, subscriber):
        config = queue_item.get('destination', {}).get('config', {})

        try:
            with ftp_connect(config) as ftp:

                if config.get('push_associated', False):
                    # Set the working directory for the associated files
                    if 'associated_path' in config and config.get('associated_path'):
                        ftp.cwd('/' + config.get('associated_path', '').lstrip('/'))

                    item = self._get_published_item(queue_item)
                    if item:
                        self._copy_published_media_files(item, ftp)

                    # If the directory was changed to push associated files change it back
                    if 'associated_path' in config and config.get('associated_path'):
                        ftp.cwd('/' + config.get('path').lstrip('/'))

                filename = get_publish_service().get_filename(queue_item)
                b = BytesIO(queue_item.get('encoded_item', queue_item.get('formatted_item').encode('UTF-8')))
                ftp.storbinary('STOR ' + filename, b)
        except PublishFtpError:
            raise
        except Exception as ex:
            raise PublishFtpError.ftpError(ex, config)

    def _copy_published_media_files(self, item, ftp):
        renditions_filter = get_renditions_filter()

        def parse_media(item):
            media = {}
            renditions = item.get('renditions', {})
            for key, rendition in renditions.items():
                if key not in renditions_filter:
                    continue
                if not rendition.get('media'):
                    logger.warn('media missing on rendition %s for item %s', key, item['guid'])
                    continue
                rendition.pop('href', None)
                rendition.setdefault('mimetype', rendition.get('original', {}).get('mimetype', item.get('mimetype')))
                media[rendition['media']] = rendition
            return media

        media = {}
        media.update(parse_media(item))

        for assoc in item.get('associations', {}).values():
            if assoc is None:
                continue
            media.update(parse_media(assoc))
            for assoc2 in assoc.get('associations', {}).values():
                if assoc2 is None:
                    continue
                media.update(parse_media(assoc2))

        # Retrieve the list of files that currently exist in the FTP server
        remote_items = []
        ftp.retrlines('LIST', remote_items.append)

        for media_id, rendition in media.items():
            if not self._media_exists(rendition, remote_items):
                binary = app.media.get(media_id, resource=rendition.get('resource', 'upload'))
                self._transmit_media(binary, rendition, ftp)

    def _media_exists(self, rendition, items):
        for file in items:
            if get_rendition_file_name(rendition) in file:
                return True
        return False

    def _transmit_media(self, binary, rendition, ftp):
        ftp.storbinary('STOR ' + get_rendition_file_name(rendition), binary)


register_transmitter('ftp', FTPPublishService(), errors)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import json

from eve.utils import ParsedRequest
from flask_babel import _

from superdesk import get_resource_service, app
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError


class ContentFilterTestResource(Resource):
    endpoint_name = 'content_filter_tests'
    schema = {
        'filter_id': {'type': 'string'},
        'article_id': {'type': 'string'},
        'return_matching': {'type': 'boolean'},
        'filter': {'type': 'dict'},
        'match_results': {'type': 'list'},
    }
    url = 'content_filters/test'
    resource_methods = ['POST']
    item_methods = []
    resource_title = endpoint_name
    privileges = {'POST': 'content_filters'}


class ContentFilterTestService(BaseService):

    def create(self, docs, **kwargs):
        service = get_resource_service('content_filters')
        for doc in docs:
            filter_id = doc.get('filter_id')
            if filter_id:
                content_filter = service.find_one(req=None, _id=filter_id)
            else:
                content_filter = doc.get('filter')

            if not content_filter:
                raise SuperdeskApiError.badRequestError(_('Content filter not found'))

            if 'article_id' in doc:
                article_id = doc.get('article_id')
                article = get_resource_service('archive').find_one(req=None, _id=article_id)
                if not article and 'planning' in app.config.get('INSTALLED_APPS', []):
                    article = get_resource_service('planning').find_one(None, _id=article_id)
                if not article:
                    article = get_resource_service('ingest').find_one(req=None, _id=article_id)
                    if not article:
                        raise SuperdeskApiError.badRequestError(_('Article not found!'))
                try:
                    doc['match_results'] = service.does_match(content_filter, article)
                except Exception as ex:
                    raise SuperdeskApiError.\
                        badRequestError(_('Error in testing article: {error}').format(error=str(ex)))
            else:
                try:
                    if doc.get('return_matching', True):
                        query = service.build_elastic_query(content_filter)
                    else:
                        query = service.build_elastic_not_filter(content_filter)
                    query['sort'] = [{'versioncreated': 'desc'}]
                    query['size'] = 200
                    req = ParsedRequest()
                    req.args = {'source': json.dumps(query)}
                    doc['match_results'] = list(get_resource_service('archive').get(req=req, lookup=None))
                except Exception as ex:
                    raise SuperdeskApiError.\
                        badRequestError(_('Error in testing archive: {error}').format(error=str(ex)))

        return [len(docs)]




import locale
import warnings
import superdesk

from flask import current_app as app, request
from flask_babel import _
from datetime import timedelta
from superdesk.utils import ListCursor
from superdesk.utc import utcnow
from superdesk.errors import SuperdeskApiError


SETTING_ENABLED = 'ARCHIVE_AUTOCOMPLETE'
SETTING_DAYS = 'ARCHIVE_AUTOCOMPLETE_DAYS'
SETTING_HOURS = 'ARCHIVE_AUTOCOMPLETE_HOURS'


class AutocompleteResource(superdesk.Resource):
    item_methods = []
    resource_methods = ['GET']
    schema = {
        'value': {'type': 'string'},
    }


class AutocompleteService(superdesk.Service):

    allowed_fields = {
        'slugline',
    }

    def get(self, req, lookup):
        field = request.args.get('field', 'slugline')
        language = request.args.get('language', app.config.get('DEFAULT_LANGUAGE', 'en'))

        if not app.config.get(SETTING_ENABLED):
            raise SuperdeskApiError(_("Archive autocomplete is not enabled"), 404)

        if field not in self.allowed_fields:
            raise SuperdeskApiError(_("Field %(field)s is not allowed", field=field), 400)

        _filter = {
            'state': 'published',
            'language': language,
            'versioncreated': {'$gte': utcnow() - timedelta(
                days=app.config[SETTING_DAYS],
                hours=app.config[SETTING_HOURS],
            )},
        }
        values = self.backend._backend('archive').driver.db['archive'].distinct(field, _filter)
        sorted_values = sorted(values, key=locale.strxfrm)
        docs = [{'value': value} for value in sorted_values]
        return ListCursor(docs)


def init_app(_app):
    _app.client_config.update({'archive_autocomplete': _app.config.get(SETTING_ENABLED, False)})

    if _app.config.get(SETTING_ENABLED) and not _app.config.get(SETTING_HOURS) and not _app.config.get(SETTING_DAYS):
        warnings.warn(
            '{} is enabled but both {} and {} are not set.'.format(
                SETTING_ENABLED,
                SETTING_HOURS,
                SETTING_DAYS,
            )
        )

    superdesk.register_resource(
        'archive_autocomplete',
        AutocompleteResource,
        AutocompleteService,
        _app=_app)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging

import superdesk

from .archive_copy import CopyService, CopyResource, init_app as archive_copy_init_app
from .archive_duplication import DuplicateService, DuplicateResource
from .archive_fetch import FetchResource, FetchService
from .archive_move import MoveResource, MoveService
from .archive_translate import TranslateService, TranslateResource


logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = 'fetch'
    service = FetchService(endpoint_name, backend=superdesk.get_backend())
    FetchResource(endpoint_name, app=app, service=service)

    endpoint_name = 'duplicate'
    service = DuplicateService(endpoint_name, backend=superdesk.get_backend())
    DuplicateResource(endpoint_name, app=app, service=service)

    endpoint_name = 'translate'
    service = TranslateService(endpoint_name, backend=superdesk.get_backend())
    TranslateResource(endpoint_name, app=app, service=service)

    endpoint_name = 'copy'
    service = CopyService(endpoint_name, backend=superdesk.get_backend())
    CopyResource(endpoint_name, app=app, service=service)

    endpoint_name = 'move'
    service = MoveService(endpoint_name, backend=superdesk.get_backend())
    MoveResource(endpoint_name, app=app, service=service)

    superdesk.privilege(name='fetch', label='Fetch Content To a Desk', description='Fetch Content to a Desk')
    superdesk.privilege(name='move', label='Move Content to another desk', description='Move Content to another desk')
    superdesk.privilege(name='duplicate', label='Duplicate Content within a Desk',
                        description='Duplicate Content within a Desk')
    superdesk.privilege(name='translate', label='Translate Content within a Desk',
                        description='Translate Content within a Desk')

    superdesk.intrinsic_privilege('copy', method=['POST'])

    archive_copy_init_app(app)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .area_acre_to_metric import acre_to_metric


class TemperatureTestCase(unittest.TestCase):

    def test_f_to_c(self):
        text = 'Total area is 100.50 ac. for this land. '\
               'Total area is 15.7 Acres for this land. '\
               'Total area is 1 acre for this land. '\
               'Total area is 1-16 acres for this land. '\
               'Total area is 16.7-acres for this land. '\
               'Total area is 16,500-acre for this land. '\

        item = {'body_html': text}
        res, diff = acre_to_metric(item)
        self.assertEqual(diff['100.50 ac'], '100.50 ac (40.7 ha)')
        self.assertEqual(diff['15.7 Acres'], '15.7 Acres (6.4 ha)')
        self.assertEqual(diff['1 acre'], '1 acre (4,047 square meter)')
        self.assertEqual(diff['1-16 acres'], '1-16 acres (0.4-6.5 ha)')
        self.assertEqual(diff['16.7-acres'], '16.7-acres (6.8 ha)')
        self.assertEqual(diff['16,500-acre'], '16,500-acre (6,677 ha)')
        self.assertEqual(res['body_html'], item['body_html'])



import unittest

import flask
import flask_mail
from email.parser import Parser
from email.header import decode_header

from superdesk.emails import SuperdeskMessage


class SuperdeskMessageTestCase(unittest.TestCase):
    subject = 'темы для выделения выделения выделения'

    def test_unicode_subject(self):
        app = flask.Flask(__name__)
        flask_mail.Mail(app)
        with app.app_context():
            msg = SuperdeskMessage(self.subject, sender='root', body='test')
            out = msg.as_bytes()
        parsed = Parser().parsestr(out.decode('utf-8'), headersonly=True)
        decoded, charset = decode_header(parsed['subject'])[0]
        self.assertEqual(self.subject, decoded.decode(charset))




import logging
from raven.contrib.flask import Sentry
from raven.contrib.celery import register_signal, register_logger_signal


SENTRY_DSN = 'SENTRY_DSN'


class SuperdeskSentry():
    """Sentry proxy that will do nothing in case sentry is not configured."""

    def __init__(self, app):
        if app.config.get(SENTRY_DSN):
            if 'verify_ssl' not in app.config[SENTRY_DSN]:
                app.config[SENTRY_DSN] += '?verify_ssl=0'
            app.config.setdefault('SENTRY_NAME', app.config.get('SERVER_DOMAIN'))
            self.sentry = Sentry(app, register_signal=False, wrap_wsgi=False, logging=True, level=logging.WARNING)
            register_logger_signal(self.sentry.client)
            register_signal(self.sentry.client)
        else:
            self.sentry = None

    def captureException(self, exc_info=None, **kwargs):
        if self.sentry:
            self.sentry.captureException(exc_info, **kwargs)

    def captureMessage(self, message, **kwargs):
        if self.sentry:
            self.sentry.captureMessage(message, **kwargs)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json

from unittest.mock import MagicMock
from datetime import timedelta

from eve.versioning import resolve_document_version
from eve.utils import ParsedRequest

from apps.archive.common import insert_into_versions, ARCHIVE
from superdesk import get_resource_service
from superdesk.tests import TestCase
from superdesk.utc import utcnow
from apps.legal_archive.commands import LegalArchiveImport


class LegalArchiveTestCase(TestCase):

    desks = [{'_id': '123', 'name': 'Sports'}]
    users = [{'_id': '123', 'username': 'test1', 'first_name': 'test', 'last_name': 'user', 'email': 'a@a.com'}]
    stages = [{'_id': '123', 'name': 'working stage', 'desk': '123'}]
    archive = [
        {
            'task': {'desk': '123', 'stage': '123'}
        },
        {
            'task': {'desk': '1234', 'stage': None, 'user': '123'}
        },
        {
            'task': {'desk': '1234', 'stage': 'dddd', 'user': 'test'}
        }
    ]

    def setUp(self):
        self.app.data.insert('desks', self.desks)
        self.app.data.insert('users', self.users)
        self.app.data.insert('stages', self.stages)

    def test_denormalize_desk_user(self):
        LegalArchiveImport()._denormalize_user_desk(self.archive[0], '')
        task = self.archive[0]['task']
        self.assertEqual(task.get('desk'), 'Sports')
        self.assertEqual(task.get('stage'), 'working stage')
        self.assertEqual(task.get('user'), '')

    def test_denormalize_not_configured_desk(self):
        LegalArchiveImport()._denormalize_user_desk(self.archive[1], '')
        task = self.archive[1]['task']
        self.assertEqual(task.get('desk'), '1234')
        self.assertEqual(task.get('stage'), None)
        self.assertEqual(task.get('user'), 'test user')

    def test_denormalize_not_configured_desk_stage_user(self):
        LegalArchiveImport()._denormalize_user_desk(self.archive[2], '')
        task = self.archive[2]['task']
        self.assertEqual(task.get('desk'), '1234')
        self.assertEqual(task.get('stage'), 'dddd')
        self.assertEqual(task.get('user'), '')


class ImportLegalArchiveCommandTestCase(TestCase):
    desks = [{'name': 'Sports'}]
    users = [{'username': 'test1', 'first_name': 'test', 'last_name': 'user', 'email': 'a@a.com'}]

    def setUp(self):
        try:
            from apps.legal_archive.commands import ImportLegalArchiveCommand
        except ImportError:
            self.fail('Could not import class under test (ImportLegalArchiveCommand).')
        else:
            self.class_under_test = ImportLegalArchiveCommand
            self.app.data.insert('desks', self.desks)
            self.app.data.insert('users', self.users)
            self.validators = [
                {
                    'schema': {},
                    'type': 'text',
                    'act': 'publish',
                    '_id': 'publish_text'
                },
                {
                    'schema': {},
                    'type': 'text',
                    'act': 'correct',
                    '_id': 'correct_text'
                },
                {
                    'schema': {},
                    'type': 'text',
                    'act': 'kill',
                    '_id': 'kill_text'
                }
            ]

            self.products = [{'_id': '1', 'name': 'prod1'},
                             {'_id': '2', 'name': 'prod2', 'codes': 'abc,def'},
                             {'_id': '3', 'name': 'prod3', 'codes': 'xyz'}]

            self.subscribers = [
                {'name': 'Test', 'is_active': True, 'subscriber_type': 'wire',
                 'email': 'test@test.com', 'sequence_num_settings': {'max': 9999, 'min': 1},
                 'products': ['1'],
                 'destinations': [{'name': 'test', 'delivery_type': 'email', 'format': 'nitf',
                                   'config': {'recipients': 'test@test.com'}}]}
            ]
            self.app.data.insert('validators', self.validators)
            self.app.data.insert('products', self.products)
            self.app.data.insert('subscribers', self.subscribers)
            self.class_under_test = ImportLegalArchiveCommand
            self.archive_items = [
                {
                    'task': {'desk': self.desks[0]['_id'], 'stage': self.desks[0]['incoming_stage'], 'user': '123'},
                    '_id': 'item1', 'state': 'in_progress', 'headline': 'item 1', 'type': 'text',
                    'slugline': 'item 1 slugline', '_current_version': 1, '_created': utcnow() - timedelta(minutes=3),
                    'expired': utcnow() - timedelta(minutes=30)
                },
                {
                    'task': {'desk': self.desks[0]['_id'], 'stage': self.desks[0]['incoming_stage'], 'user': '123'},
                    '_id': 'item2', 'state': 'in_progress', 'headline': 'item 2', 'type': 'text',
                    'slugline': 'item 2 slugline', '_current_version': 1, '_created': utcnow() - timedelta(minutes=2),
                    'expired': utcnow() - timedelta(minutes=30)
                },
                {
                    'task': {'desk': self.desks[0]['_id'], 'stage': self.desks[0]['incoming_stage'], 'user': '123'},
                    '_id': 'item3', 'state': 'in_progress', 'headline': 'item 2', 'type': 'text',
                    'slugline': 'item 2 slugline', '_current_version': 1, '_created': utcnow() - timedelta(minutes=1),
                    'expired': utcnow() - timedelta(minutes=30)
                }
            ]

            get_resource_service(ARCHIVE).post(self.archive_items)
            for item in self.archive_items:
                resolve_document_version(item, ARCHIVE, 'POST')
                insert_into_versions(id_=item['_id'])

    def test_import_into_legal_archive(self):
        archive_publish = get_resource_service('archive_publish')
        archive_correct = get_resource_service('archive_correct')
        legal_archive = get_resource_service('legal_archive')
        archive = get_resource_service('archive_publish')
        published = get_resource_service('published')
        publish_queue = get_resource_service('publish_queue')

        self.original_method = LegalArchiveImport.upsert_into_legal_archive
        LegalArchiveImport.upsert_into_legal_archive = MagicMock()

        for item in self.archive_items:
            archive_publish.patch(item['_id'], {'headline': 'publishing', 'abstract': 'publishing'})

        for item in self.archive_items:
            legal_item = legal_archive.find_one(req=None, _id=item['_id'])
            self.assertIsNone(legal_item, 'Item: {} is not none.'.format(item['_id']))

        archive_correct.patch(self.archive_items[1]['_id'], {'headline': 'correcting', 'abstract': 'correcting'})

        LegalArchiveImport.upsert_into_legal_archive = self.original_method
        self.class_under_test().run(1)

        # items are not expired
        for item in self.archive_items:
            legal_item = legal_archive.find_one(req=None, _id=item['_id'])
            self.assertIsNone(legal_item, 'Item: {} is not none.'.format(item['_id']))

        # expire the items
        for item in self.archive_items:
            original = archive.find_one(req=None, _id=item['_id'])
            archive.system_update(item['_id'], {'expiry': utcnow() - timedelta(minutes=30)}, original)
            published.update_published_items(item['_id'], 'expiry', utcnow() - timedelta(minutes=30))

        # run the command after expiry
        self.class_under_test().run(1)

        # items are expired
        for item in self.archive_items:
            legal_item = legal_archive.find_one(req=None, _id=item['_id'])
            self.assertEqual(legal_item['_id'], item['_id'], 'item {} not imported to legal'.format(item['_id']))

        # items are moved to legal
        for item in self.archive_items:
            published_items = list(published.get_other_published_items(item['_id']))
            for published_item in published_items:
                self.assertEqual(published_item['moved_to_legal'], True)

        # items are moved to legal publish queue
        for item in self.archive_items:
            req = ParsedRequest()
            req.where = json.dumps({'item_id': item['_id']})
            queue_items = list(publish_queue.get(req=req, lookup=None))
            self.assertGreaterEqual(len(queue_items), 1)
            for queue_item in queue_items:
                self.assertEqual(queue_item['moved_to_legal'], True)




import superdesk

from superdesk import get_backend
from apps.highlights.resource import HighlightsResource, MarkedForHighlightsResource
from apps.highlights.service import HighlightsService, MarkedForHighlightsService
from .generate import GenerateHighlightsService, GenerateHighlightsResource


def init_app(app):
    endpoint_name = 'highlights'
    service = HighlightsService(endpoint_name, backend=get_backend())
    HighlightsResource(endpoint_name, app=app, service=service)

    endpoint_name = 'marked_for_highlights'
    service = MarkedForHighlightsService(endpoint_name, backend=get_backend())
    MarkedForHighlightsResource(endpoint_name, app=app, service=service)

    endpoint_name = 'generate_highlights'
    service = GenerateHighlightsService(endpoint_name, backend=get_backend())
    GenerateHighlightsResource(endpoint_name, app=app, service=service)

    superdesk.privilege(name='highlights_read', label='Highlights - read',
                        description='Can see a list of highlights.')
    superdesk.privilege(name='highlights', label='Highlights/Summary List Management',
                        description='User can manage highlights/summary lists.')
    superdesk.privilege(name='mark_for_highlights', label='Mark items for Highlights/Summary Lists',
                        description='User can mark items for Highlights/Summary Lists.')
